@@ -1,25 +1,27 @@
 // test the throughput of the fs.WriteStream class.
 'use strict';
 
-var path = require('path');
-var common = require('../common.js');
-var filename = path.resolve(__dirname, '.removeme-benchmark-garbage');
-var fs = require('fs');
-var filesize = 1000 * 1024 * 1024;
-var assert = require('assert');
+const path = require('path');
+const common = require('../common.js');
+const filename = path.resolve(process.env.NODE_TMPDIR || __dirname,
+                              `.removeme-benchmark-garbage-${process.pid}`);
+const fs = require('fs');
+const assert = require('assert');
 
-var type, encoding, size;
+let encodingType, encoding, size, filesize;
 
-var bench = common.createBenchmark(main, {
-  type: ['buf', 'asc', 'utf'],
+const bench = common.createBenchmark(main, {
+  encodingType: ['buf', 'asc', 'utf'],
+  filesize: [1000 * 1024 * 1024],
   size: [1024, 4096, 65535, 1024 * 1024]
 });
 
 function main(conf) {
-  type = conf.type;
+  encodingType = conf.encodingType;
   size = +conf.size;
+  filesize = conf.filesize;
 
-  switch (type) {
+  switch (encodingType) {
     case 'buf':
       encoding = null;
       break;
@@ -30,15 +32,15 @@ function main(conf) {
       encoding = 'utf8';
       break;
     default:
-      throw new Error('invalid type');
+      throw new Error(`invalid encodingType: ${encodingType}`);
   }
 
-  makeFile(runTest);
+  makeFile();
 }
 
 function runTest() {
   assert(fs.statSync(filename).size === filesize);
-  var rs = fs.createReadStream(filename, {
+  const rs = fs.createReadStream(filename, {
     highWaterMark: size,
     encoding: encoding
   });
@@ -60,7 +62,7 @@ function runTest() {
 }
 
 function makeFile() {
-  var buf = new Buffer(filesize / 1024);
+  const buf = Buffer.allocUnsafe(filesize / 1024);
   if (encoding === 'utf8') {
     // ü
     for (var i = 0; i < buf.length; i++) {
@@ -74,7 +76,7 @@ function makeFile() {
 
   try { fs.unlinkSync(filename); } catch (e) {}
   var w = 1024;
-  var ws = fs.createWriteStream(filename);
+  const ws = fs.createWriteStream(filename);
   ws.on('close', runTest);
   ws.on('drain', write);
   write();
