@@ -1,19 +1,21 @@
 'use strict';
 
-var common = require('../common.js');
-var FreeList = require('internal/freelist').FreeList;
+const common = require('../common.js');
 
-var bench = common.createBenchmark(main, {
+const bench = common.createBenchmark(main, {
   n: [100000]
+}, {
+  flags: ['--expose-internals']
 });
 
 function main(conf) {
-  var n = conf.n;
-  var poolSize = 1000;
-  var list = new FreeList('test', poolSize, Object);
+  const FreeList = require('internal/freelist');
+  const n = conf.n;
+  const poolSize = 1000;
+  const list = new FreeList('test', poolSize, Object);
   var i;
   var j;
-  var used = [];
+  const used = [];
 
   // First, alloc `poolSize` items
   for (j = 0; j < poolSize; j++) {
