@@ -4,8 +4,8 @@
 // Note that JS speed goes up, while cxx speed stays about the same.
 'use strict';
 
-var assert = require('assert');
-var common = require('../../common.js');
+const assert = require('assert');
+const common = require('../../common.js');
 
 // this fails when we try to open with a different version of node,
 // which is quite common for benchmarks.  so in that case, just
@@ -17,7 +17,7 @@ try {
   console.error('misc/function_call.js Binding failed to load');
   process.exit(0);
 }
-var cxx = binding.hello;
+const cxx = binding.hello;
 
 var c = 0;
 function js() {
@@ -26,15 +26,15 @@ function js() {
 
 assert(js() === cxx());
 
-var bench = common.createBenchmark(main, {
+const bench = common.createBenchmark(main, {
   type: ['js', 'cxx'],
   millions: [1, 10, 50]
 });
 
 function main(conf) {
-  var n = +conf.millions * 1e6;
+  const n = +conf.millions * 1e6;
 
-  var fn = conf.type === 'cxx' ? cxx : js;
+  const fn = conf.type === 'cxx' ? cxx : js;
   bench.start();
   for (var i = 0; i < n; i++) {
     fn();
