@@ -1,9 +1,8 @@
 'use strict';
-var common = require('../common.js');
-var path = require('path');
-var v8 = require('v8');
+const common = require('../common.js');
+const path = require('path');
 
-var bench = common.createBenchmark(main, {
+const bench = common.createBenchmark(main, {
   pathext: [
     '',
     '/',
@@ -20,22 +19,16 @@ var bench = common.createBenchmark(main, {
 });
 
 function main(conf) {
-  var n = +conf.n;
-  var p = path.posix;
-  var input = '' + conf.pathext;
+  const n = +conf.n;
+  const p = path.posix;
+  var input = String(conf.pathext);
   var ext;
-  var extIdx = input.indexOf('|');
+  const extIdx = input.indexOf('|');
   if (extIdx !== -1) {
     ext = input.slice(extIdx + 1);
     input = input.slice(0, extIdx);
   }
 
-  // Force optimization before starting the benchmark
-  p.basename(input, ext);
-  v8.setFlagsFromString('--allow_natives_syntax');
-  eval('%OptimizeFunctionOnNextCall(p.basename)');
-  p.basename(input, ext);
-
   bench.start();
   for (var i = 0; i < n; i++) {
     p.basename(input, ext);
