@@ -8,6 +8,8 @@
 #include "src/compiler/opcodes.h"
 #include "src/compiler/operator.h"
 #include "src/compiler/types.h"
+#include "src/handles-inl.h"
+#include "src/objects-inl.h"
 #include "src/objects/map.h"
 #include "src/objects/name.h"
 
@@ -29,63 +31,6 @@ std::ostream& operator<<(std::ostream& os, BaseTaggedness base_taggedness) {
   UNREACHABLE();
 }
 
-
-MachineType BufferAccess::machine_type() const {
-  switch (external_array_type_) {
-    case kExternalUint8Array:
-    case kExternalUint8ClampedArray:
-      return MachineType::Uint8();
-    case kExternalInt8Array:
-      return MachineType::Int8();
-    case kExternalUint16Array:
-      return MachineType::Uint16();
-    case kExternalInt16Array:
-      return MachineType::Int16();
-    case kExternalUint32Array:
-      return MachineType::Uint32();
-    case kExternalInt32Array:
-      return MachineType::Int32();
-    case kExternalFloat32Array:
-      return MachineType::Float32();
-    case kExternalFloat64Array:
-      return MachineType::Float64();
-  }
-  UNREACHABLE();
-}
-
-
-bool operator==(BufferAccess lhs, BufferAccess rhs) {
-  return lhs.external_array_type() == rhs.external_array_type();
-}
-
-
-bool operator!=(BufferAccess lhs, BufferAccess rhs) { return !(lhs == rhs); }
-
-
-size_t hash_value(BufferAccess access) {
-  return base::hash<ExternalArrayType>()(access.external_array_type());
-}
-
-
-std::ostream& operator<<(std::ostream& os, BufferAccess access) {
-  switch (access.external_array_type()) {
-#define TYPED_ARRAY_CASE(Type, type, TYPE, ctype, size) \
-  case kExternal##Type##Array:                          \
-    return os << #Type;
-    TYPED_ARRAYS(TYPED_ARRAY_CASE)
-#undef TYPED_ARRAY_CASE
-  }
-  UNREACHABLE();
-}
-
-
-BufferAccess const BufferAccessOf(const Operator* op) {
-  DCHECK(op->opcode() == IrOpcode::kLoadBuffer ||
-         op->opcode() == IrOpcode::kStoreBuffer);
-  return OpParameter<BufferAccess>(op);
-}
-
-
 bool operator==(FieldAccess const& lhs, FieldAccess const& rhs) {
   // On purpose we don't include the write barrier kind here, as this method is
   // really only relevant for eliminating loads and they don't care about the
@@ -270,6 +215,11 @@ CheckMapsParameters const& CheckMapsParametersOf(Operator const* op) {
   return OpParameter<CheckMapsParameters>(op);
 }
 
+ZoneHandleSet<Map> const& CompareMapsParametersOf(Operator const* op) {
+  DCHECK_EQ(IrOpcode::kCompareMaps, op->opcode());
+  return OpParameter<ZoneHandleSet<Map>>(op);
+}
+
 size_t hash_value(CheckTaggedInputMode mode) {
   return static_cast<size_t>(mode);
 }
@@ -401,6 +351,8 @@ std::ostream& operator<<(std::ostream& os, NumberOperationHint hint) {
   switch (hint) {
     case NumberOperationHint::kSignedSmall:
       return os << "SignedSmall";
+    case NumberOperationHint::kSignedSmallInputs:
+      return os << "SignedSmallInputs";
     case NumberOperationHint::kSigned32:
       return os << "Signed32";
     case NumberOperationHint::kNumber:
@@ -430,7 +382,9 @@ NumberOperationHint NumberOperationHintOf(const Operator* op) {
          op->opcode() == IrOpcode::kSpeculativeNumberBitwiseXor ||
          op->opcode() == IrOpcode::kSpeculativeNumberEqual ||
          op->opcode() == IrOpcode::kSpeculativeNumberLessThan ||
-         op->opcode() == IrOpcode::kSpeculativeNumberLessThanOrEqual);
+         op->opcode() == IrOpcode::kSpeculativeNumberLessThanOrEqual ||
+         op->opcode() == IrOpcode::kSpeculativeSafeIntegerAdd ||
+         op->opcode() == IrOpcode::kSpeculativeSafeIntegerSubtract);
   return OpParameter<NumberOperationHint>(op);
 }
 
@@ -546,6 +500,7 @@ UnicodeEncoding UnicodeEncodingOf(const Operator* op) {
   V(TruncateTaggedPointerToBit, Operator::kNoProperties, 1, 0)   \
   V(TruncateTaggedToWord32, Operator::kNoProperties, 1, 0)       \
   V(TruncateTaggedToFloat64, Operator::kNoProperties, 1, 0)      \
+  V(ObjectIsCallable, Operator::kNoProperties, 1, 0)             \
   V(ObjectIsDetectableCallable, Operator::kNoProperties, 1, 0)   \
   V(ObjectIsNaN, Operator::kNoProperties, 1, 0)                  \
   V(ObjectIsNonCallable, Operator::kNoProperties, 1, 0)          \
@@ -577,7 +532,6 @@ UnicodeEncoding UnicodeEncodingOf(const Operator* op) {
   V(CheckSmi, 1, 1)                    \
   V(CheckString, 1, 1)                 \
   V(CheckSeqString, 1, 1)              \
-  V(CheckNonEmptyString, 1, 1)         \
   V(CheckSymbol, 1, 1)                 \
   V(CheckNotTaggedHole, 1, 1)          \
   V(CheckedInt32Add, 2, 1)             \
@@ -746,6 +700,16 @@ struct SimplifiedOperatorGlobalCache final {
   CheckedTruncateTaggedToWord32Operator<CheckTaggedInputMode::kNumberOrOddball>
       kCheckedTruncateTaggedToWord32NumberOrOddballOperator;
 
+  struct CheckMapValueOperator final : public Operator {
+    CheckMapValueOperator()
+        : Operator(                                     // --
+              IrOpcode::kCheckMapValue,                 // opcode
+              Operator::kNoThrow | Operator::kNoWrite,  // flags
+              "CheckMapValue",                          // name
+              2, 1, 1, 0, 1, 0) {}                      // counts
+  };
+  CheckMapValueOperator kCheckMapValue;
+
   template <CheckFloat64HoleMode kMode>
   struct CheckFloat64HoleNaNOperator final
       : public Operator1<CheckFloat64HoleMode> {
@@ -780,6 +744,8 @@ struct SimplifiedOperatorGlobalCache final {
   };                                                                        \
   Name##Operator<NumberOperationHint::kSignedSmall>                         \
       k##Name##SignedSmallOperator;                                         \
+  Name##Operator<NumberOperationHint::kSignedSmallInputs>                   \
+      k##Name##SignedSmallInputsOperator;                                   \
   Name##Operator<NumberOperationHint::kSigned32> k##Name##Signed32Operator; \
   Name##Operator<NumberOperationHint::kNumber> k##Name##NumberOperator;     \
   Name##Operator<NumberOperationHint::kNumberOrOddball>                     \
@@ -806,37 +772,13 @@ struct SimplifiedOperatorGlobalCache final {
       kSpeculativeToNumberNumberOperator;
   SpeculativeToNumberOperator<NumberOperationHint::kNumberOrOddball>
       kSpeculativeToNumberNumberOrOddballOperator;
-
-#define BUFFER_ACCESS(Type, type, TYPE, ctype, size)                          \
-  struct LoadBuffer##Type##Operator final : public Operator1<BufferAccess> {  \
-    LoadBuffer##Type##Operator()                                              \
-        : Operator1<BufferAccess>(                                            \
-              IrOpcode::kLoadBuffer,                                          \
-              Operator::kNoDeopt | Operator::kNoThrow | Operator::kNoWrite,   \
-              "LoadBuffer", 3, 1, 1, 1, 1, 0,                                 \
-              BufferAccess(kExternal##Type##Array)) {}                        \
-  };                                                                          \
-  struct StoreBuffer##Type##Operator final : public Operator1<BufferAccess> { \
-    StoreBuffer##Type##Operator()                                             \
-        : Operator1<BufferAccess>(                                            \
-              IrOpcode::kStoreBuffer,                                         \
-              Operator::kNoDeopt | Operator::kNoRead | Operator::kNoThrow,    \
-              "StoreBuffer", 4, 1, 1, 0, 1, 0,                                \
-              BufferAccess(kExternal##Type##Array)) {}                        \
-  };                                                                          \
-  LoadBuffer##Type##Operator kLoadBuffer##Type;                               \
-  StoreBuffer##Type##Operator kStoreBuffer##Type;
-  TYPED_ARRAYS(BUFFER_ACCESS)
-#undef BUFFER_ACCESS
 };
 
-
-static base::LazyInstance<SimplifiedOperatorGlobalCache>::type kCache =
-    LAZY_INSTANCE_INITIALIZER;
-
+static base::LazyInstance<SimplifiedOperatorGlobalCache>::type
+    kSimplifiedOperatorGlobalCache = LAZY_INSTANCE_INITIALIZER;
 
 SimplifiedOperatorBuilder::SimplifiedOperatorBuilder(Zone* zone)
-    : cache_(kCache.Get()), zone_(zone) {}
+    : cache_(kSimplifiedOperatorGlobalCache.Get()), zone_(zone) {}
 
 #define GET_FROM_CACHE(Name, ...) \
   const Operator* SimplifiedOperatorBuilder::Name() { return &cache_.k##Name; }
@@ -846,6 +788,7 @@ GET_FROM_CACHE(ArrayBufferWasNeutered)
 GET_FROM_CACHE(ArgumentsFrame)
 GET_FROM_CACHE(LookupHashStorageIndex)
 GET_FROM_CACHE(LoadHashMapValue)
+GET_FROM_CACHE(CheckMapValue)
 GET_FROM_CACHE(NewUnmappedArgumentsElements)
 #undef GET_FROM_CACHE
 
@@ -926,6 +869,16 @@ const Operator* SimplifiedOperatorBuilder::CheckMaps(CheckMapsFlags flags,
       parameters);                                     // parameter
 }
 
+const Operator* SimplifiedOperatorBuilder::CompareMaps(
+    ZoneHandleSet<Map> maps) {
+  return new (zone()) Operator1<ZoneHandleSet<Map>>(  // --
+      IrOpcode::kCompareMaps,                         // opcode
+      Operator::kEliminatable,                        // flags
+      "CompareMaps",                                  // name
+      1, 1, 1, 1, 1, 0,                               // counts
+      maps);                                          // parameter
+}
+
 const Operator* SimplifiedOperatorBuilder::CheckFloat64Hole(
     CheckFloat64HoleMode mode) {
   switch (mode) {
@@ -942,6 +895,8 @@ const Operator* SimplifiedOperatorBuilder::SpeculativeToNumber(
   switch (hint) {
     case NumberOperationHint::kSignedSmall:
       return &cache_.kSpeculativeToNumberSignedSmallOperator;
+    case NumberOperationHint::kSignedSmallInputs:
+      break;
     case NumberOperationHint::kSigned32:
       return &cache_.kSpeculativeToNumberSigned32Operator;
     case NumberOperationHint::kNumber:
@@ -1029,30 +984,6 @@ const Operator* SimplifiedOperatorBuilder::Allocate(Type* type,
       1, 1, 1, 1, 1, 0, AllocateParameters(type, pretenure));
 }
 
-
-const Operator* SimplifiedOperatorBuilder::LoadBuffer(BufferAccess access) {
-  switch (access.external_array_type()) {
-#define LOAD_BUFFER(Type, type, TYPE, ctype, size) \
-  case kExternal##Type##Array:                     \
-    return &cache_.kLoadBuffer##Type;
-    TYPED_ARRAYS(LOAD_BUFFER)
-#undef LOAD_BUFFER
-  }
-  UNREACHABLE();
-}
-
-
-const Operator* SimplifiedOperatorBuilder::StoreBuffer(BufferAccess access) {
-  switch (access.external_array_type()) {
-#define STORE_BUFFER(Type, type, TYPE, ctype, size) \
-  case kExternal##Type##Array:                      \
-    return &cache_.kStoreBuffer##Type;
-    TYPED_ARRAYS(STORE_BUFFER)
-#undef STORE_BUFFER
-  }
-  UNREACHABLE();
-}
-
 const Operator* SimplifiedOperatorBuilder::StringFromCodePoint(
     UnicodeEncoding encoding) {
   switch (encoding) {
@@ -1069,6 +1000,8 @@ const Operator* SimplifiedOperatorBuilder::StringFromCodePoint(
     switch (hint) {                                                           \
       case NumberOperationHint::kSignedSmall:                                 \
         return &cache_.k##Name##SignedSmallOperator;                          \
+      case NumberOperationHint::kSignedSmallInputs:                           \
+        return &cache_.k##Name##SignedSmallInputsOperator;                    \
       case NumberOperationHint::kSigned32:                                    \
         return &cache_.k##Name##Signed32Operator;                             \
       case NumberOperationHint::kNumber:                                      \
