@@ -4,6 +4,7 @@
 
 #include "src/safepoint-table.h"
 
+#include "src/assembler-inl.h"
 #include "src/deoptimizer.h"
 #include "src/disasm.h"
 #include "src/frames-inl.h"
@@ -35,26 +36,45 @@ bool SafepointEntry::HasRegisterAt(int reg_index) const {
 
 
 SafepointTable::SafepointTable(Code* code) {
-  DCHECK(code->is_crankshafted());
+  DCHECK(code->is_turbofanned());
   code_ = code;
   Address header = code->instruction_start() + code->safepoint_table_offset();
   length_ = Memory::uint32_at(header + kLengthOffset);
   entry_size_ = Memory::uint32_at(header + kEntrySizeOffset);
   pc_and_deoptimization_indexes_ = header + kHeaderSize;
-  entries_ = pc_and_deoptimization_indexes_ +
-             (length_ * kPcAndDeoptimizationIndexSize);
+  entries_ = pc_and_deoptimization_indexes_ + (length_ * kFixedEntrySize);
   DCHECK(entry_size_ > 0);
   STATIC_ASSERT(SafepointEntry::DeoptimizationIndexField::kMax ==
                 Safepoint::kNoDeoptimizationIndex);
 }
 
+unsigned SafepointTable::find_return_pc(unsigned pc_offset) {
+  for (unsigned i = 0; i < length(); i++) {
+    if (GetTrampolinePcOffset(i) == static_cast<int>(pc_offset)) {
+      return GetPcOffset(i);
+    } else if (GetPcOffset(i) == pc_offset) {
+      return pc_offset;
+    }
+  }
+  UNREACHABLE();
+  return 0;
+}
 
 SafepointEntry SafepointTable::FindEntry(Address pc) const {
   unsigned pc_offset = static_cast<unsigned>(pc - code_->instruction_start());
-  for (unsigned i = 0; i < length(); i++) {
+  // We use kMaxUInt32 as sentinel value, so check that we don't hit that.
+  DCHECK_NE(kMaxUInt32, pc_offset);
+  unsigned len = length();
+  // If pc == kMaxUInt32, then this entry covers all call sites in the function.
+  if (len == 1 && GetPcOffset(0) == kMaxUInt32) return GetEntry(0);
+  for (unsigned i = 0; i < len; i++) {
     // TODO(kasperl): Replace the linear search with binary search.
-    if (GetPcOffset(i) == pc_offset) return GetEntry(i);
+    if (GetPcOffset(i) == pc_offset ||
+        GetTrampolinePcOffset(i) == static_cast<int>(pc_offset)) {
+      return GetEntry(i);
+    }
   }
+  UNREACHABLE();
   return SafepointEntry();
 }
 
@@ -109,6 +129,7 @@ Safepoint SafepointTableBuilder::DefineSafepoint(
   info.pc = assembler->pc_offset();
   info.arguments = arguments;
   info.has_doubles = (kind & Safepoint::kWithDoubles);
+  info.trampoline = -1;
   deoptimization_info_.Add(info, zone_);
   deopt_index_list_.Add(Safepoint::kNoDeoptimizationIndex, zone_);
   if (deopt_mode == Safepoint::kNoLazyDeopt) {
@@ -134,8 +155,24 @@ unsigned SafepointTableBuilder::GetCodeOffset() const {
   return offset_;
 }
 
+int SafepointTableBuilder::UpdateDeoptimizationInfo(int pc, int trampoline,
+                                                    int start) {
+  int index = -1;
+  for (int i = start; i < deoptimization_info_.length(); i++) {
+    if (static_cast<int>(deoptimization_info_[i].pc) == pc) {
+      index = i;
+      break;
+    }
+  }
+  CHECK(index >= 0);
+  DCHECK(index < deoptimization_info_.length());
+  deoptimization_info_[index].trampoline = trampoline;
+  return index;
+}
 
 void SafepointTableBuilder::Emit(Assembler* assembler, int bits_per_entry) {
+  RemoveDuplicates();
+
   // Make sure the safepoint table is properly aligned. Pad with nops.
   assembler->Align(kIntSize);
   assembler->RecordComment(";;; Safepoint table.");
@@ -158,6 +195,7 @@ void SafepointTableBuilder::Emit(Assembler* assembler, int bits_per_entry) {
     assembler->dd(deoptimization_info_[i].pc);
     assembler->dd(EncodeExceptPC(deoptimization_info_[i],
                                  deopt_index_list_[i]));
+    assembler->dd(deoptimization_info_[i].trampoline);
   }
 
   // Emit table of bitmaps.
@@ -210,6 +248,63 @@ uint32_t SafepointTableBuilder::EncodeExceptPC(const DeoptimizationInfo& info,
   return encoding;
 }
 
+void SafepointTableBuilder::RemoveDuplicates() {
+  // If the table contains more than one entry, and all entries are identical
+  // (except for the pc), replace the whole table by a single entry with pc =
+  // kMaxUInt32. This especially compacts the table for wasm code without tagged
+  // pointers and without deoptimization info.
+
+  int length = deoptimization_info_.length();
+  DCHECK_EQ(length, deopt_index_list_.length());
+  DCHECK_EQ(length, indexes_.length());
+  DCHECK_EQ(length, registers_.length());
+
+  if (length < 2) return;
+
+  // Check that all entries (1, length] are identical to entry 0.
+  for (int i = 1; i < length; ++i) {
+    if (!IsIdenticalExceptForPc(0, i)) return;
+  }
+
+  // If we get here, all entries were identical. Rewind all lists to just one
+  // entry, and set the pc to kMaxUInt32.
+  deoptimization_info_.Rewind(1);
+  deopt_index_list_.Rewind(1);
+  indexes_.Rewind(1);
+  registers_.Rewind(1);
+  deoptimization_info_[0].pc = kMaxUInt32;
+}
+
+bool SafepointTableBuilder::IsIdenticalExceptForPc(int index1,
+                                                   int index2) const {
+  DeoptimizationInfo& deopt_info_1 = deoptimization_info_[index1];
+  DeoptimizationInfo& deopt_info_2 = deoptimization_info_[index2];
+  if (deopt_info_1.arguments != deopt_info_2.arguments) return false;
+  if (deopt_info_1.has_doubles != deopt_info_2.has_doubles) return false;
+
+  if (deopt_index_list_[index1] != deopt_index_list_[index2]) return false;
+
+  ZoneList<int>* indexes1 = indexes_[index1];
+  ZoneList<int>* indexes2 = indexes_[index2];
+  if (indexes1->length() != indexes2->length()) return false;
+  for (int i = 0; i < indexes1->length(); ++i) {
+    if (indexes1->at(i) != indexes2->at(i)) return false;
+  }
+
+  ZoneList<int>* registers1 = registers_[index1];
+  ZoneList<int>* registers2 = registers_[index2];
+  if (registers1) {
+    if (!registers2) return false;
+    if (registers1->length() != registers2->length()) return false;
+    for (int i = 0; i < registers1->length(); ++i) {
+      if (registers1->at(i) != registers2->at(i)) return false;
+    }
+  } else if (registers2) {
+    return false;
+  }
+
+  return true;
+}
 
 }  // namespace internal
 }  // namespace v8
