@@ -1,26 +1,93 @@
+// Copyright Joyent, Inc. and other Node contributors.
+//
+// Permission is hereby granted, free of charge, to any person obtaining a
+// copy of this software and associated documentation files (the
+// "Software"), to deal in the Software without restriction, including
+// without limitation the rights to use, copy, modify, merge, publish,
+// distribute, sublicense, and/or sell copies of the Software, and to permit
+// persons to whom the Software is furnished to do so, subject to the
+// following conditions:
+//
+// The above copyright notice and this permission notice shall be included
+// in all copies or substantial portions of the Software.
+//
+// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
+// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
+// NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
+// DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
+// OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
+// USE OR OTHER DEALINGS IN THE SOFTWARE.
+
 'use strict';
 
 const util = require('util');
 const net = require('net');
 const url = require('url');
-const HTTPParser = process.binding('http_parser').HTTPParser;
+const { HTTPParser } = process.binding('http_parser');
 const assert = require('assert').ok;
-const common = require('_http_common');
-const httpSocketSetup = common.httpSocketSetup;
-const parsers = common.parsers;
-const freeParser = common.freeParser;
-const debug = common.debug;
-const OutgoingMessage = require('_http_outgoing').OutgoingMessage;
+const {
+  _checkIsHttpToken: checkIsHttpToken,
+  debug,
+  freeParser,
+  httpSocketSetup,
+  parsers
+} = require('_http_common');
+const { OutgoingMessage } = require('_http_outgoing');
 const Agent = require('_http_agent');
-const Buffer = require('buffer').Buffer;
+const { Buffer } = require('buffer');
+const { urlToOptions, searchParamsSymbol } = require('internal/url');
+const { outHeadersKey } = require('internal/http');
+const { nextTick } = require('internal/process/next_tick');
 
+// The actual list of disallowed characters in regexp form is more like:
+//    /[^A-Za-z0-9\-._~!$&'()*+,;=/:@]/
+// with an additional rule for ignoring percentage-escaped characters, but
+// that's a) hard to capture in a regular expression that performs well, and
+// b) possibly too restrictive for real-world usage. So instead we restrict the
+// filter to just control characters and spaces.
+//
+// This function is used in the case of small paths, where manual character code
+// checks can greatly outperform the equivalent regexp (tested in V8 5.4).
+function isInvalidPath(s) {
+  var i = 0;
+  if (s.charCodeAt(0) <= 32) return true;
+  if (++i >= s.length) return false;
+  if (s.charCodeAt(1) <= 32) return true;
+  if (++i >= s.length) return false;
+  if (s.charCodeAt(2) <= 32) return true;
+  if (++i >= s.length) return false;
+  if (s.charCodeAt(3) <= 32) return true;
+  if (++i >= s.length) return false;
+  if (s.charCodeAt(4) <= 32) return true;
+  if (++i >= s.length) return false;
+  if (s.charCodeAt(5) <= 32) return true;
+  ++i;
+  for (; i < s.length; ++i)
+    if (s.charCodeAt(i) <= 32) return true;
+  return false;
+}
+
+function validateHost(host, name) {
+  if (host != null && typeof host !== 'string') {
+    throw new TypeError(
+      `"options.${name}" must either be a string, undefined or null`);
+  }
+  return host;
+}
 
 function ClientRequest(options, cb) {
-  var self = this;
-  OutgoingMessage.call(self);
+  OutgoingMessage.call(this);
 
   if (typeof options === 'string') {
     options = url.parse(options);
+    if (!options.hostname) {
+      throw new Error('Unable to determine the domain name');
+    }
+  } else if (options && options[searchParamsSymbol] &&
+             options[searchParamsSymbol][searchParamsSymbol]) {
+    // url.URL instance
+    options = urlToOptions(options);
   } else {
     options = util._extend({}, options);
   }
@@ -29,66 +96,93 @@ function ClientRequest(options, cb) {
   var defaultAgent = options._defaultAgent || Agent.globalAgent;
   if (agent === false) {
     agent = new defaultAgent.constructor();
-  } else if ((agent === null || agent === undefined) &&
-             !options.createConnection) {
-    agent = defaultAgent;
+  } else if (agent === null || agent === undefined) {
+    if (typeof options.createConnection !== 'function') {
+      agent = defaultAgent;
+    }
+    // Explicitly pass through this statement as agent will not be used
+    // when createConnection is provided.
+  } else if (typeof agent.addRequest !== 'function') {
+    throw new TypeError(
+      'Agent option must be an Agent-like object, undefined, or false.'
+    );
   }
-  self.agent = agent;
+  this.agent = agent;
 
   var protocol = options.protocol || defaultAgent.protocol;
   var expectedProtocol = defaultAgent.protocol;
-  if (self.agent && self.agent.protocol)
-    expectedProtocol = self.agent.protocol;
+  if (this.agent && this.agent.protocol)
+    expectedProtocol = this.agent.protocol;
 
-  if (options.path && /[\u0000-\u0020]/.test(options.path)) {
-    // The actual regex is more like /[^A-Za-z0-9\-._~!$&'()*+,;=/:@]/
-    // with an additional rule for ignoring percentage-escaped characters
-    // but that's a) hard to capture in a regular expression that performs
-    // well, and b) possibly too restrictive for real-world usage. That's
-    // why it only scans for spaces because those are guaranteed to create
-    // an invalid request.
-    throw new TypeError('Request path contains unescaped characters.');
-  } else if (protocol !== expectedProtocol) {
-    throw new Error('Protocol "' + protocol + '" not supported. ' +
-                    'Expected "' + expectedProtocol + '".');
+  var path;
+  if (options.path) {
+    path = String(options.path);
+    var invalidPath;
+    if (path.length <= 39) { // Determined experimentally in V8 5.4
+      invalidPath = isInvalidPath(path);
+    } else {
+      invalidPath = /[\u0000-\u0020]/.test(path);
+    }
+    if (invalidPath)
+      throw new TypeError('Request path contains unescaped characters');
   }
 
-  const defaultPort = options.defaultPort ||
-                      self.agent && self.agent.defaultPort;
+  if (protocol !== expectedProtocol) {
+    throw new Error('Protocol "' + protocol + '" not supported. ' +
+                    'Expected "' + expectedProtocol + '"');
+  }
+
+  var defaultPort = options.defaultPort ||
+                    this.agent && this.agent.defaultPort;
 
   var port = options.port = options.port || defaultPort || 80;
-  var host = options.host = options.hostname || options.host || 'localhost';
+  var host = options.host = validateHost(options.hostname, 'hostname') ||
+                            validateHost(options.host, 'host') || 'localhost';
 
-  if (options.setHost === undefined) {
-    var setHost = true;
+  var setHost = (options.setHost === undefined);
+
+  this.socketPath = options.socketPath;
+  this.timeout = options.timeout;
+
+  var method = options.method;
+  var methodIsString = (typeof method === 'string');
+  if (method != null && !methodIsString) {
+    throw new TypeError('Method must be a string');
   }
 
-  self.socketPath = options.socketPath;
+  if (methodIsString && method) {
+    if (!checkIsHttpToken(method)) {
+      throw new TypeError('Method must be a valid HTTP token');
+    }
+    method = this.method = method.toUpperCase();
+  } else {
+    method = this.method = 'GET';
+  }
 
-  var method = self.method = (options.method || 'GET').toUpperCase();
-  self.path = options.path || '/';
+  this.path = options.path || '/';
   if (cb) {
-    self.once('response', cb);
+    this.once('response', cb);
   }
 
-  if (!Array.isArray(options.headers)) {
+  var headersArray = Array.isArray(options.headers);
+  if (!headersArray) {
     if (options.headers) {
       var keys = Object.keys(options.headers);
-      for (var i = 0, l = keys.length; i < l; i++) {
+      for (var i = 0; i < keys.length; i++) {
         var key = keys[i];
-        self.setHeader(key, options.headers[key]);
+        this.setHeader(key, options.headers[key]);
       }
     }
     if (host && !this.getHeader('host') && setHost) {
       var hostHeader = host;
-      var posColon = -1;
 
       // For the Host header, ensure that IPv6 addresses are enclosed
       // in square brackets, as defined by URI formatting
       // https://tools.ietf.org/html/rfc3986#section-3.2.2
-      if (-1 !== (posColon = hostHeader.indexOf(':')) &&
-          -1 !== (posColon = hostHeader.indexOf(':', posColon)) &&
-          '[' !== hostHeader[0]) {
+      var posColon = hostHeader.indexOf(':');
+      if (posColon !== -1 &&
+          hostHeader.indexOf(':', posColon + 1) !== -1 &&
+          hostHeader.charCodeAt(0) !== 91/*'['*/) {
         hostHeader = `[${hostHeader}]`;
       }
 
@@ -100,9 +194,8 @@ function ClientRequest(options, cb) {
   }
 
   if (options.auth && !this.getHeader('Authorization')) {
-    //basic auth
     this.setHeader('Authorization', 'Basic ' +
-                   new Buffer(options.auth).toString('base64'));
+                   Buffer.from(options.auth).toString('base64'));
   }
 
   if (method === 'GET' ||
@@ -110,77 +203,101 @@ function ClientRequest(options, cb) {
       method === 'DELETE' ||
       method === 'OPTIONS' ||
       method === 'CONNECT') {
-    self.useChunkedEncodingByDefault = false;
+    this.useChunkedEncodingByDefault = false;
   } else {
-    self.useChunkedEncodingByDefault = true;
+    this.useChunkedEncodingByDefault = true;
   }
 
-  if (Array.isArray(options.headers)) {
-    self._storeHeader(self.method + ' ' + self.path + ' HTTP/1.1\r\n',
+  if (headersArray) {
+    this._storeHeader(this.method + ' ' + this.path + ' HTTP/1.1\r\n',
                       options.headers);
-  } else if (self.getHeader('expect')) {
-    self._storeHeader(self.method + ' ' + self.path + ' HTTP/1.1\r\n',
-                      self._renderHeaders());
+  } else if (this.getHeader('expect')) {
+    if (this._header) {
+      throw new Error('Can\'t render headers after they are sent to the ' +
+                      'client');
+    }
+
+    this._storeHeader(this.method + ' ' + this.path + ' HTTP/1.1\r\n',
+                      this[outHeadersKey]);
   }
 
-  if (self.socketPath) {
-    self._last = true;
-    self.shouldKeepAlive = false;
-    self.onSocket(self.agent.createConnection({ path: self.socketPath }));
-  } else if (self.agent) {
+  this._ended = false;
+  this.res = null;
+  this.aborted = undefined;
+  this.timeoutCb = null;
+  this.upgradeOrConnect = false;
+  this.parser = null;
+  this.maxHeadersCount = null;
+
+  var called = false;
+
+  var oncreate = (err, socket) => {
+    if (called)
+      return;
+    called = true;
+    if (err) {
+      process.nextTick(() => this.emit('error', err));
+      return;
+    }
+    this.onSocket(socket);
+    this._deferToConnect(null, null, () => this._flush());
+  };
+
+  if (this.agent) {
     // If there is an agent we should default to Connection:keep-alive,
     // but only if the Agent will actually reuse the connection!
     // If it's not a keepAlive agent, and the maxSockets==Infinity, then
     // there's never a case where this socket will actually be reused
-    if (!self.agent.keepAlive && !Number.isFinite(self.agent.maxSockets)) {
-      self._last = true;
-      self.shouldKeepAlive = false;
+    if (!this.agent.keepAlive && !Number.isFinite(this.agent.maxSockets)) {
+      this._last = true;
+      this.shouldKeepAlive = false;
     } else {
-      self._last = false;
-      self.shouldKeepAlive = true;
+      this._last = false;
+      this.shouldKeepAlive = true;
     }
-    self.agent.addRequest(self, options);
+    this.agent.addRequest(this, options);
   } else {
     // No agent, default to Connection:close.
-    self._last = true;
-    self.shouldKeepAlive = false;
-    if (options.createConnection) {
-      self.onSocket(options.createConnection(options));
+    this._last = true;
+    this.shouldKeepAlive = false;
+    if (typeof options.createConnection === 'function') {
+      const newSocket = options.createConnection(options, oncreate);
+      if (newSocket && !called) {
+        called = true;
+        this.onSocket(newSocket);
+      } else {
+        return;
+      }
     } else {
       debug('CLIENT use net.createConnection', options);
-      self.onSocket(net.createConnection(options));
+      this.onSocket(net.createConnection(options));
     }
   }
 
-  self._deferToConnect(null, null, function() {
-    self._flush();
-    self = null;
-  });
-
-  this._ended = false;
+  this._deferToConnect(null, null, () => this._flush());
 }
 
 util.inherits(ClientRequest, OutgoingMessage);
 
-exports.ClientRequest = ClientRequest;
 
-ClientRequest.prototype.aborted = undefined;
-
-ClientRequest.prototype._finish = function() {
+ClientRequest.prototype._finish = function _finish() {
   DTRACE_HTTP_CLIENT_REQUEST(this, this.connection);
   LTTNG_HTTP_CLIENT_REQUEST(this, this.connection);
   COUNTER_HTTP_CLIENT_REQUEST();
   OutgoingMessage.prototype._finish.call(this);
 };
 
-ClientRequest.prototype._implicitHeader = function() {
+ClientRequest.prototype._implicitHeader = function _implicitHeader() {
+  if (this._header) {
+    throw new Error('Can\'t render headers after they are sent to the client');
+  }
   this._storeHeader(this.method + ' ' + this.path + ' HTTP/1.1\r\n',
-                    this._renderHeaders());
+                    this[outHeadersKey]);
 };
 
-ClientRequest.prototype.abort = function() {
-  if (this.aborted === undefined) {
-    process.nextTick(emitAbortNT, this);
+ClientRequest.prototype.abort = function abort() {
+  if (!this.aborted) {
+    process.nextTick(emitAbortNT.bind(this));
   }
   // Mark as aborting so we can avoid sending queued request data
   // This is used as a truthy flag elsewhere. The use of Date.now is for
@@ -188,12 +305,13 @@ ClientRequest.prototype.abort = function() {
   this.aborted = Date.now();
 
   // If we're aborting, we don't care about any more response data.
-  if (this.res)
+  if (this.res) {
     this.res._dump();
-  else
+  } else {
     this.once('response', function(res) {
       res._dump();
     });
+  }
 
   // In the event that we don't have a socket, we will pop out of
   // the request queue through handling in onSocket.
@@ -204,8 +322,8 @@ ClientRequest.prototype.abort = function() {
 };
 
 
-function emitAbortNT(self) {
-  self.emit('abort');
+function emitAbortNT() {
+  this.emit('abort');
 }
 
 
@@ -321,7 +439,7 @@ function socketOnData(d) {
 
   var ret = parser.execute(d);
   if (ret instanceof Error) {
-    debug('parse error');
+    debug('parse error', ret);
     freeParser(parser, req, socket);
     socket.destroy();
     req.emit('error', ret);
@@ -408,7 +526,7 @@ function parserOnIncomingClient(res, shouldKeepAlive) {
 
   if (res.statusCode === 100) {
     // restart the parser, as this is a continue message.
-    delete req.res; // Clear res so that we don't hit double-responses.
+    req.res = null; // Clear res so that we don't hit double-responses.
     req.emit('continue');
     return true;
   }
@@ -448,7 +566,10 @@ function responseKeepAlive(res, req) {
   if (!req.shouldKeepAlive) {
     if (socket.writable) {
       debug('AGENT socket.destroySoon()');
-      socket.destroySoon();
+      if (typeof socket.destroySoon === 'function')
+        socket.destroySoon();
+      else
+        socket.end();
     }
     assert(!socket.writable);
   } else {
@@ -460,9 +581,12 @@ function responseKeepAlive(res, req) {
     socket.removeListener('close', socketCloseListener);
     socket.removeListener('error', socketErrorListener);
     socket.once('error', freeSocketErrorListener);
+    // There are cases where _handle === null. Avoid those. Passing null to
+    // nextTick() will call getDefaultTriggerAsyncId() to retrieve the id.
+    const asyncId = socket._handle ? socket._handle.getAsyncId() : null;
     // Mark this socket as available, AFTER user-added end
     // handlers have a chance to run.
-    process.nextTick(emitFreeNT, socket);
+    nextTick(asyncId, emitFreeNT, socket);
   }
 }
 
@@ -520,17 +644,27 @@ function tickOnSocket(req, socket) {
   socket.on('data', socketOnData);
   socket.on('end', socketOnEnd);
   socket.on('close', socketCloseListener);
+
+  if (req.timeout) {
+    const emitRequestTimeout = () => req.emit('timeout');
+    socket.once('timeout', emitRequestTimeout);
+    req.once('response', (res) => {
+      res.once('end', () => {
+        socket.removeListener('timeout', emitRequestTimeout);
+      });
+    });
+  }
   req.emit('socket', socket);
 }
 
-ClientRequest.prototype.onSocket = function(socket) {
+ClientRequest.prototype.onSocket = function onSocket(socket) {
   process.nextTick(onSocketNT, this, socket);
 };
 
 function onSocketNT(req, socket) {
   if (req.aborted) {
     // If we were aborted while waiting for a socket, skip the whole thing.
-    if (req.socketPath || !req.agent) {
+    if (!req.agent) {
       socket.destroy();
     } else {
       socket.emit('free');
@@ -540,44 +674,41 @@ function onSocketNT(req, socket) {
   }
 }
 
-ClientRequest.prototype._deferToConnect = function(method, arguments_, cb) {
+ClientRequest.prototype._deferToConnect = _deferToConnect;
+function _deferToConnect(method, arguments_, cb) {
   // This function is for calls that need to happen once the socket is
   // connected and writable. It's an important promisy thing for all the socket
   // calls that happen either now (when a socket is assigned) or
   // in the future (when a socket gets assigned out of the pool and is
   // eventually writable).
-  var self = this;
 
-  function callSocketMethod() {
+  const callSocketMethod = () => {
     if (method)
-      self.socket[method].apply(self.socket, arguments_);
+      this.socket[method].apply(this.socket, arguments_);
 
     if (typeof cb === 'function')
       cb();
-  }
+  };
 
-  var onSocket = function() {
-    if (self.socket.writable) {
+  const onSocket = () => {
+    if (this.socket.writable) {
       callSocketMethod();
     } else {
-      self.socket.once('connect', callSocketMethod);
+      this.socket.once('connect', callSocketMethod);
     }
   };
 
-  if (!self.socket) {
-    self.once('socket', onSocket);
+  if (!this.socket) {
+    this.once('socket', onSocket);
   } else {
     onSocket();
   }
-};
+}
 
-ClientRequest.prototype.setTimeout = function(msecs, callback) {
+ClientRequest.prototype.setTimeout = function setTimeout(msecs, callback) {
   if (callback) this.once('timeout', callback);
 
-  var self = this;
-  function emitTimeout() {
-    self.emit('timeout');
-  }
+  const emitTimeout = () => this.emit('timeout');
 
   if (this.socket && this.socket.writable) {
     if (this.timeoutCb)
@@ -604,21 +735,19 @@ ClientRequest.prototype.setTimeout = function(msecs, callback) {
   return this;
 };
 
-ClientRequest.prototype.setNoDelay = function() {
-  const argsLen = arguments.length;
-  const args = new Array(argsLen);
-  for (var i = 0; i < argsLen; i++)
-    args[i] = arguments[i];
-  this._deferToConnect('setNoDelay', args);
-};
-ClientRequest.prototype.setSocketKeepAlive = function() {
-  const argsLen = arguments.length;
-  const args = new Array(argsLen);
-  for (var i = 0; i < argsLen; i++)
-    args[i] = arguments[i];
-  this._deferToConnect('setKeepAlive', args);
+ClientRequest.prototype.setNoDelay = function setNoDelay(noDelay) {
+  this._deferToConnect('setNoDelay', [noDelay]);
 };
 
-ClientRequest.prototype.clearTimeout = function(cb) {
+ClientRequest.prototype.setSocketKeepAlive =
+    function setSocketKeepAlive(enable, initialDelay) {
+      this._deferToConnect('setKeepAlive', [enable, initialDelay]);
+    };
+
+ClientRequest.prototype.clearTimeout = function clearTimeout(cb) {
   this.setTimeout(0, cb);
 };
+
+module.exports = {
+  ClientRequest
+};
