@@ -1,17 +1,40 @@
+// Copyright Joyent, Inc. and other Node contributors.
+//
+// Permission is hereby granted, free of charge, to any person obtaining a
+// copy of this software and associated documentation files (the
+// "Software"), to deal in the Software without restriction, including
+// without limitation the rights to use, copy, modify, merge, publish,
+// distribute, sublicense, and/or sell copies of the Software, and to permit
+// persons to whom the Software is furnished to do so, subject to the
+// following conditions:
+//
+// The above copyright notice and this permission notice shall be included
+// in all copies or substantial portions of the Software.
+//
+// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
+// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
+// NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
+// DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
+// OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
+// USE OR OTHER DEALINGS IN THE SOFTWARE.
+
 'use strict';
 
-require('internal/util').assertCrypto();
+const internalUtil = require('internal/util');
+internalUtil.assertCrypto();
 
 const assert = require('assert');
+const { Buffer } = require('buffer');
+const common = require('_tls_common');
+const { Connection } = process.binding('crypto');
 const EventEmitter = require('events');
 const stream = require('stream');
+const { Timer } = process.binding('timer_wrap');
 const tls = require('tls');
 const util = require('util');
-const common = require('_tls_common');
+
 const debug = util.debuglog('tls-legacy');
-const Buffer = require('buffer').Buffer;
-const Timer = process.binding('timer_wrap').Timer;
-const Connection = process.binding('crypto').Connection;
 
 function SlabBuffer() {
   this.create();
@@ -479,6 +502,7 @@ CryptoStream.prototype._done = function() {
 
 
 // readyState is deprecated. Don't use it.
+// Deprecation Code: DEP0004
 Object.defineProperty(CryptoStream.prototype, 'readyState', {
   get: function() {
     if (this.connecting) {
@@ -608,6 +632,8 @@ function onhandshakestart() {
     // state machine and OpenSSL is not re-entrant. We cannot allow the user's
     // callback to destroy the connection right now, it would crash and burn.
     setImmediate(function() {
+      // Old-style error is not being migrated to the newer style
+      // internal/errors.js because _tls_legacy.js has been deprecated.
       var err = new Error('TLS session renegotiation attack detected');
       if (self.cleartext) self.cleartext.emit('error', err);
     });
@@ -787,18 +813,11 @@ function securePairNT(self, options) {
 }
 
 
-exports.createSecurePair = function(context,
-                                    isServer,
-                                    requestCert,
-                                    rejectUnauthorized,
-                                    options) {
-  var pair = new SecurePair(context,
-                            isServer,
-                            requestCert,
-                            rejectUnauthorized,
-                            options);
-  return pair;
-};
+function createSecurePair(context, isServer, requestCert,
+                          rejectUnauthorized, options) {
+  return new SecurePair(context, isServer, requestCert,
+                        rejectUnauthorized, options);
+}
 
 
 SecurePair.prototype.maybeInitFinished = function() {
@@ -861,14 +880,14 @@ SecurePair.prototype.error = function(returnOnly) {
              /peer did not return a certificate/.test(err.message)) {
     // Not really an error.
     this.destroy();
-  } else {
-    if (!returnOnly) this.cleartext.emit('error', err);
+  } else if (!returnOnly) {
+    this.cleartext.emit('error', err);
   }
   return err;
 };
 
 
-exports.pipe = function pipe(pair, socket) {
+function pipe(pair, socket) {
   pair.encrypted.pipe(socket);
   socket.pipe(pair.encrypted);
 
@@ -918,7 +937,7 @@ exports.pipe = function pipe(pair, socket) {
   socket.on('timeout', ontimeout);
 
   return cleartext;
-};
+}
 
 
 function pipeCloseNT(pair, socket) {
@@ -927,3 +946,11 @@ function pipeCloseNT(pair, socket) {
   pair.encrypted.unpipe(socket);
   socket.destroySoon();
 }
+
+module.exports = {
+  createSecurePair:
+    internalUtil.deprecate(createSecurePair,
+                           'tls.createSecurePair() is deprecated. Please use ' +
+                           'tls.Socket instead.', 'DEP0064'),
+  pipe
+};
