@@ -1,34 +1,42 @@
+// Copyright Joyent, Inc. and other Node contributors.
+//
+// Permission is hereby granted, free of charge, to any person obtaining a
+// copy of this software and associated documentation files (the
+// "Software"), to deal in the Software without restriction, including
+// without limitation the rights to use, copy, modify, merge, publish,
+// distribute, sublicense, and/or sell copies of the Software, and to permit
+// persons to whom the Software is furnished to do so, subject to the
+// following conditions:
+//
+// The above copyright notice and this permission notice shall be included
+// in all copies or substantial portions of the Software.
+//
+// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
+// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
+// NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
+// DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
+// OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
+// USE OR OTHER DEALINGS IN THE SOFTWARE.
+
 'use strict';
 
+require('internal/util').assertCrypto();
+
 const assert = require('assert');
 const crypto = require('crypto');
 const net = require('net');
 const tls = require('tls');
 const util = require('util');
 const common = require('_tls_common');
-const StreamWrap = require('_stream_wrap').StreamWrap;
-const Buffer = require('buffer').Buffer;
-const Duplex = require('stream').Duplex;
+const { StreamWrap } = require('_stream_wrap');
+const { Buffer } = require('buffer');
 const debug = util.debuglog('tls');
-const Timer = process.binding('timer_wrap').Timer;
+const { Timer } = process.binding('timer_wrap');
 const tls_wrap = process.binding('tls_wrap');
-const TCP = process.binding('tcp_wrap').TCP;
-const Pipe = process.binding('pipe_wrap').Pipe;
-const defaultSessionIdContext = getDefaultSessionIdContext();
-
-function getDefaultSessionIdContext() {
-  var defaultText = process.argv.join(' ');
-  /* SSL_MAX_SID_CTX_LENGTH is 128 bits */
-  if (process.binding('config').fipsMode) {
-    return crypto.createHash('sha1')
-      .update(defaultText)
-      .digest('hex').slice(0, 32);
-  } else {
-    return crypto.createHash('md5')
-      .update(defaultText)
-      .digest('hex');
-  }
-}
+const { TCP, constants: TCPConstants } = process.binding('tcp_wrap');
+const { Pipe, constants: PipeConstants } = process.binding('pipe_wrap');
+const kDisableRenegotiation = Symbol('disable-renegotiation');
 
 function onhandshakestart() {
   debug('onhandshakestart');
@@ -52,10 +60,15 @@ function onhandshakestart() {
     // state machine and OpenSSL is not re-entrant. We cannot allow the user's
     // callback to destroy the connection right now, it would crash and burn.
     setImmediate(function() {
-      var err = new Error('TLS session renegotiation attack detected.');
+      var err = new Error('TLS session renegotiation attack detected');
       self._emitTLSError(err);
     });
   }
+
+  if (this[kDisableRenegotiation] && ssl.handshakes > 0) {
+    const err = new Error('TLS session renegotiation disabled for this socket');
+    self._emitTLSError(err);
+  }
 }
 
 
@@ -79,12 +92,8 @@ function loadSession(self, hello, cb) {
     if (!self._handle)
       return cb(new Error('Socket is closed'));
 
-    // NOTE: That we have disabled OpenSSL's internal session storage in
-    // `node_crypto.cc` and hence its safe to rely on getting servername only
-    // from clienthello or this place.
-    var ret = self._handle.loadSession(session);
-
-    cb(null, ret);
+    self._handle.loadSession(session);
+    cb(null);
   }
 
   if (hello.sessionId.length <= 0 ||
@@ -168,7 +177,7 @@ function requestOCSP(self, hello, ctx, cb) {
 function onclienthello(hello) {
   var self = this;
 
-  loadSession(self, hello, function(err, session) {
+  loadSession(self, hello, function(err) {
     if (err)
       return self.destroy(err);
 
@@ -271,12 +280,10 @@ function TLSSocket(socket, options) {
 
   // Wrap plain JS Stream into StreamWrap
   var wrap;
-  if (!(socket instanceof net.Socket) && socket instanceof Duplex)
-    wrap = new StreamWrap(socket);
-  else if ((socket instanceof net.Socket) && !socket._handle)
-    wrap = new StreamWrap(socket);
-  else
+  if ((socket instanceof net.Socket && socket._handle) || !socket)
     wrap = socket;
+  else
+    wrap = new StreamWrap(socket);
 
   // Just a documented property to make secure sockets
   // distinguishable from regular ones.
@@ -292,11 +299,11 @@ function TLSSocket(socket, options) {
   // Proxy for API compatibility
   this.ssl = this._handle;
 
-  this.on('error', this._emitTLSError);
+  this.on('error', this._tlsError);
 
   this._init(socket, wrap);
 
-  // Make sure to setup all required properties like: `_connecting` before
+  // Make sure to setup all required properties like: `connecting` before
   // starting the flow of the data
   this.readable = true;
   this.writable = true;
@@ -317,12 +324,16 @@ var proxiedMethods = [
 ];
 
 // Proxy HandleWrap, PipeWrap and TCPWrap methods
-proxiedMethods.forEach(function(name) {
-  tls_wrap.TLSWrap.prototype[name] = function methodProxy() {
+function makeMethodProxy(name) {
+  return function methodProxy(...args) {
     if (this._parent[name])
-      return this._parent[name].apply(this._parent, arguments);
+      return this._parent[name].apply(this._parent, args);
   };
-});
+}
+for (var n = 0; n < proxiedMethods.length; n++) {
+  tls_wrap.TLSWrap.prototype[proxiedMethods[n]] =
+    makeMethodProxy(proxiedMethods[n]);
+}
 
 tls_wrap.TLSWrap.prototype.close = function close(cb) {
   let ssl;
@@ -352,6 +363,10 @@ tls_wrap.TLSWrap.prototype.close = function close(cb) {
   return this._parent.close(done);
 };
 
+TLSSocket.prototype.disableRenegotiation = function disableRenegotiation() {
+  this[kDisableRenegotiation] = true;
+};
+
 TLSSocket.prototype._wrapHandle = function(wrap) {
   var res;
   var handle;
@@ -361,14 +376,16 @@ TLSSocket.prototype._wrapHandle = function(wrap) {
 
   var options = this._tlsOptions;
   if (!handle) {
-    handle = options.pipe ? new Pipe() : new TCP();
+    handle = options.pipe ?
+      new Pipe(PipeConstants.SOCKET) :
+      new TCP(TCPConstants.SOCKET);
     handle.owner = this;
   }
 
   // Wrap socket's handle
   var context = options.secureContext ||
                 options.credentials ||
-                tls.createSecureContext();
+                tls.createSecureContext(options);
   res = tls_wrap.wrap(handle._externalStream,
                       context.context,
                       !!options.isServer);
@@ -414,9 +431,8 @@ TLSSocket.prototype._init = function(socket, wrap) {
   var ssl = this._handle;
 
   // lib/net.js expect this value to be non-zero if write hasn't been flushed
-  // immediately
-  // TODO(indutny): rewise this solution, it might be 1 before handshake and
-  // represent real writeQueueSize during regular writes.
+  // immediately. After the handshake is done this will represent the actual
+  // write queue size
   ssl.writeQueueSize = 1;
 
   this.server = options.server;
@@ -463,7 +479,9 @@ TLSSocket.prototype._init = function(socket, wrap) {
 
     // Destroy socket if error happened before handshake's finish
     if (!self._secureEstablished) {
-      self.destroy(self._tlsError(err));
+      // When handshake fails control is not yet released,
+      // so self._tlsError will return null instead of actual error
+      self.destroy(err);
     } else if (options.isServer &&
                rejectUnauthorized &&
                /peer did not return a certificate/.test(err.message)) {
@@ -507,9 +525,9 @@ TLSSocket.prototype._init = function(socket, wrap) {
     this._parent = socket;
 
     // To prevent assertion in afterConnect() and properly kick off readStart
-    this._connecting = socket._connecting || !socket._handle;
+    this.connecting = socket.connecting || !socket._handle;
     socket.once('connect', function() {
-      self._connecting = false;
+      self.connecting = false;
       self.emit('connect');
     });
   }
@@ -521,7 +539,7 @@ TLSSocket.prototype._init = function(socket, wrap) {
     });
   } else {
     assert(!socket);
-    this._connecting = true;
+    this.connecting = true;
   }
 };
 
@@ -563,7 +581,7 @@ TLSSocket.prototype.renegotiate = function(options, callback) {
 };
 
 TLSSocket.prototype.setMaxSendFragment = function setMaxSendFragment(size) {
-  return this._handle.setMaxSendFragment(size) == 1;
+  return this._handle.setMaxSendFragment(size) === 1;
 };
 
 TLSSocket.prototype.getTLSTicket = function getTLSTicket() {
@@ -591,7 +609,7 @@ TLSSocket.prototype._releaseControl = function() {
   if (this._controlReleased)
     return false;
   this._controlReleased = true;
-  this.removeListener('error', this._emitTLSError);
+  this.removeListener('error', this._tlsError);
   return true;
 };
 
@@ -622,7 +640,7 @@ TLSSocket.prototype._finishInit = function() {
 };
 
 TLSSocket.prototype._start = function() {
-  if (this._connecting) {
+  if (this.connecting) {
     this.once('connect', function() {
       this._start();
     });
@@ -645,14 +663,14 @@ TLSSocket.prototype.setServername = function(name) {
 
 TLSSocket.prototype.setSession = function(session) {
   if (typeof session === 'string')
-    session = new Buffer(session, 'binary');
+    session = Buffer.from(session, 'latin1');
   this._handle.setSession(session);
 };
 
 TLSSocket.prototype.getPeerCertificate = function(detailed) {
   if (this._handle) {
     return common.translatePeerCertificate(
-        this._handle.getPeerCertificate(detailed));
+      this._handle.getPeerCertificate(detailed));
   }
 
   return null;
@@ -682,6 +700,20 @@ TLSSocket.prototype.getCipher = function(err) {
   }
 };
 
+TLSSocket.prototype.getEphemeralKeyInfo = function() {
+  if (this._handle)
+    return this._handle.getEphemeralKeyInfo();
+
+  return null;
+};
+
+TLSSocket.prototype.getProtocol = function() {
+  if (this._handle)
+    return this._handle.getProtocol();
+
+  return null;
+};
+
 // TODO: support anonymous (nocert) and PSK
 
 
@@ -702,7 +734,7 @@ TLSSocket.prototype.getCipher = function(err) {
 // outcomes:
 //
 //   A) verifyError returns null meaning the client's certificate is signed
-//   by one of the server's CAs. The server know's the client idenity now
+//   by one of the server's CAs. The server now knows the client's identity
 //   and the client is authorized.
 //
 //   B) For some reason the client's certificate is not acceptable -
@@ -749,18 +781,19 @@ TLSSocket.prototype.getCipher = function(err) {
 //   "PATH_LENGTH_EXCEEDED", "INVALID_PURPOSE" "CERT_UNTRUSTED",
 //   "CERT_REJECTED"
 //
-function Server(/* [options], listener */) {
-  var options, listener;
+function Server(options, listener) {
+  if (!(this instanceof Server))
+    return new Server(options, listener);
 
-  if (arguments[0] !== null && typeof arguments[0] === 'object') {
-    options = arguments[0];
-    listener = arguments[1];
-  } else if (typeof arguments[0] === 'function') {
+  if (typeof options === 'function') {
+    listener = options;
     options = {};
-    listener = arguments[0];
+  } else if (options == null || typeof options === 'object') {
+    options = options || {};
+  } else {
+    throw new TypeError('options must be an object');
   }
 
-  if (!(this instanceof Server)) return new Server(options, listener);
 
   this._contexts = [];
 
@@ -842,14 +875,14 @@ function Server(/* [options], listener */) {
         errorEmitted = true;
         var connReset = new Error('socket hang up');
         connReset.code = 'ECONNRESET';
-        self.emit('clientError', connReset, socket);
+        self.emit('tlsClientError', connReset, socket);
       }
     });
 
     socket.on('_tlsError', function(err) {
       if (!socket._controlReleased && !errorEmitted) {
         errorEmitted = true;
-        self.emit('clientError', err, socket);
+        self.emit('tlsClientError', err, socket);
       }
     });
   });
@@ -874,7 +907,7 @@ Server.prototype._getServerData = function() {
 
 
 Server.prototype._setServerData = function(data) {
-  this.setTicketKeys(new Buffer(data.ticketKeys, 'hex'));
+  this.setTicketKeys(Buffer.from(data.ticketKeys, 'hex'));
 };
 
 
@@ -889,17 +922,8 @@ Server.prototype.setTicketKeys = function setTicketKeys(keys) {
 
 
 Server.prototype.setOptions = function(options) {
-  if (typeof options.requestCert === 'boolean') {
-    this.requestCert = options.requestCert;
-  } else {
-    this.requestCert = false;
-  }
-
-  if (typeof options.rejectUnauthorized === 'boolean') {
-    this.rejectUnauthorized = options.rejectUnauthorized;
-  } else {
-    this.rejectUnauthorized = false;
-  }
+  this.requestCert = options.requestCert === true;
+  this.rejectUnauthorized = options.rejectUnauthorized !== false;
 
   if (options.pfx) this.pfx = options.pfx;
   if (options.key) this.key = options.key;
@@ -926,14 +950,17 @@ Server.prototype.setOptions = function(options) {
   if (options.sessionIdContext) {
     this.sessionIdContext = options.sessionIdContext;
   } else {
-    this.sessionIdContext = defaultSessionIdContext;
+    this.sessionIdContext = crypto.createHash('sha1')
+                                  .update(process.argv.join(' '))
+                                  .digest('hex')
+                                  .slice(0, 32);
   }
 };
 
 // SNI Contexts High-Level API
 Server.prototype.addContext = function(servername, context) {
   if (!servername) {
-    throw new Error('Servername is required parameter for Server.addContext');
+    throw new Error('"servername" is required parameter for Server.addContext');
   }
 
   var re = new RegExp('^' +
@@ -947,7 +974,7 @@ function SNICallback(servername, callback) {
   var ctx;
 
   this.server._contexts.some(function(elem) {
-    if (servername.match(elem[0]) !== null) {
+    if (elem[0].test(servername)) {
       ctx = elem[1];
       return true;
     }
@@ -972,24 +999,25 @@ function SNICallback(servername, callback) {
 //
 //
 function normalizeConnectArgs(listArgs) {
-  var args = net._normalizeConnectArgs(listArgs);
+  var args = net._normalizeArgs(listArgs);
   var options = args[0];
   var cb = args[1];
 
+  // If args[0] was options, then normalize dealt with it.
+  // If args[0] is port, or args[0], args[1] is host, port, we need to
+  // find the options and merge them in, normalize's options has only
+  // the host/port/path args that it knows about, not the tls options.
+  // This means that options.host overrides a host arg.
   if (listArgs[1] !== null && typeof listArgs[1] === 'object') {
-    options = util._extend(options, listArgs[1]);
+    util._extend(options, listArgs[1]);
   } else if (listArgs[2] !== null && typeof listArgs[2] === 'object') {
-    options = util._extend(options, listArgs[2]);
+    util._extend(options, listArgs[2]);
   }
 
   return (cb) ? [options, cb] : [options];
 }
 
-exports.connect = function(/* [port, host], options, cb */) {
-  const argsLen = arguments.length;
-  var args = new Array(argsLen);
-  for (var i = 0; i < argsLen; i++)
-    args[i] = arguments[i];
+exports.connect = function(...args /* [port,] [host,] [options,] [cb] */) {
   args = normalizeConnectArgs(args);
   var options = args[0];
   var cb = args[1];
@@ -997,7 +1025,8 @@ exports.connect = function(/* [port, host], options, cb */) {
   var defaults = {
     rejectUnauthorized: '0' !== process.env.NODE_TLS_REJECT_UNAUTHORIZED,
     ciphers: tls.DEFAULT_CIPHERS,
-    checkServerIdentity: tls.checkServerIdentity
+    checkServerIdentity: tls.checkServerIdentity,
+    minDHSize: 1024
   };
 
   options = util._extend(defaults, options || {});
@@ -1005,23 +1034,28 @@ exports.connect = function(/* [port, host], options, cb */) {
     options.singleUse = true;
 
   assert(typeof options.checkServerIdentity === 'function');
+  assert(typeof options.minDHSize === 'number',
+         'options.minDHSize is not a number: ' + options.minDHSize);
+  assert(options.minDHSize > 0,
+         'options.minDHSize is not a positive number: ' +
+         options.minDHSize);
 
   var hostname = options.servername ||
-      options.host ||
-      (options.socket && options.socket._host) ||
-      'localhost';
-  var NPN = {};
-  var ALPN = {};
-  var context = options.secureContext || tls.createSecureContext(options);
+                 options.host ||
+                 (options.socket && options.socket._host) ||
+                 'localhost';
+  const NPN = {};
+  const ALPN = {};
+  const context = options.secureContext || tls.createSecureContext(options);
   tls.convertNPNProtocols(options.NPNProtocols, NPN);
   tls.convertALPNProtocols(options.ALPNProtocols, ALPN);
 
   var socket = new TLSSocket(options.socket, {
-    pipe: options.path && !options.port,
+    pipe: !!options.path,
     secureContext: context,
     isServer: false,
     requestCert: true,
-    rejectUnauthorized: options.rejectUnauthorized,
+    rejectUnauthorized: options.rejectUnauthorized !== false,
     session: options.session,
     NPNProtocols: NPN.NPNProtocols,
     ALPNProtocols: ALPN.ALPNProtocols,
@@ -1032,18 +1066,15 @@ exports.connect = function(/* [port, host], options, cb */) {
     socket.once('secureConnect', cb);
 
   if (!options.socket) {
-    var connect_opt;
-    if (options.path && !options.port) {
-      connect_opt = { path: options.path };
-    } else {
-      connect_opt = {
-        port: options.port,
-        host: options.host,
-        family: options.family,
-        localAddress: options.localAddress
-      };
-    }
-    socket.connect(connect_opt, function() {
+    const connectOpt = {
+      path: options.path,
+      port: options.port,
+      host: options.host,
+      family: options.family,
+      localAddress: options.localAddress,
+      lookup: options.lookup
+    };
+    socket.connect(connectOpt, function() {
       socket._start();
     });
   }
@@ -1060,6 +1091,17 @@ exports.connect = function(/* [port, host], options, cb */) {
     socket._start();
 
   socket.on('secure', function() {
+    // Check the size of DHE parameter above minimum requirement
+    // specified in options.
+    var ekeyinfo = socket.getEphemeralKeyInfo();
+    if (ekeyinfo.type === 'DH' && ekeyinfo.size < options.minDHSize) {
+      var err = new Error('DH parameter size ' + ekeyinfo.size +
+                          ' is less than ' + options.minDHSize);
+      socket.emit('error', err);
+      socket.destroy();
+      return;
+    }
+
     var verifyError = socket._handle.verifyError();
 
     // Verify that server's identity matches it's certificate's names
@@ -1094,6 +1136,10 @@ exports.connect = function(/* [port, host], options, cb */) {
       socket._hadError = true;
       var error = new Error('socket hang up');
       error.code = 'ECONNRESET';
+      error.path = options.path;
+      error.host = options.host;
+      error.port = options.port;
+      error.localAddress = options.localAddress;
       socket.destroy(error);
     }
   }
