@@ -1,3 +1,24 @@
+// Copyright Joyent, Inc. and other Node contributors.
+//
+// Permission is hereby granted, free of charge, to any person obtaining a
+// copy of this software and associated documentation files (the
+// "Software"), to deal in the Software without restriction, including
+// without limitation the rights to use, copy, modify, merge, publish,
+// distribute, sublicense, and/or sell copies of the Software, and to permit
+// persons to whom the Software is furnished to do so, subject to the
+// following conditions:
+//
+// The above copyright notice and this permission notice shall be included
+// in all copies or substantial portions of the Software.
+//
+// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
+// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
+// NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
+// DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
+// OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
+// USE OR OTHER DEALINGS IN THE SOFTWARE.
+
 'use strict';
 
 const inspect = require('util').inspect;
@@ -11,6 +32,7 @@ function assertPath(path) {
 // Resolves . and .. elements in a path with directory names
 function normalizeStringWin32(path, allowAboveRoot) {
   var res = '';
+  var lastSegmentLength = 0;
   var lastSlash = -1;
   var dots = 0;
   var code;
@@ -25,27 +47,26 @@ function normalizeStringWin32(path, allowAboveRoot) {
       if (lastSlash === i - 1 || dots === 1) {
         // NOOP
       } else if (lastSlash !== i - 1 && dots === 2) {
-        if (res.length < 2 ||
+        if (res.length < 2 || lastSegmentLength !== 2 ||
             res.charCodeAt(res.length - 1) !== 46/*.*/ ||
             res.charCodeAt(res.length - 2) !== 46/*.*/) {
           if (res.length > 2) {
-            const start = res.length - 1;
-            var j = start;
-            for (; j >= 0; --j) {
-              if (res.charCodeAt(j) === 92/*\*/)
-                break;
-            }
-            if (j !== start) {
-              if (j === -1)
+            const lastSlashIndex = res.lastIndexOf('\\');
+            if (lastSlashIndex !== res.length - 1) {
+              if (lastSlashIndex === -1) {
                 res = '';
-              else
-                res = res.slice(0, j);
+                lastSegmentLength = 0;
+              } else {
+                res = res.slice(0, lastSlashIndex);
+                lastSegmentLength = res.length - 1 - res.lastIndexOf('\\');
+              }
               lastSlash = i;
               dots = 0;
               continue;
             }
           } else if (res.length === 2 || res.length === 1) {
             res = '';
+            lastSegmentLength = 0;
             lastSlash = i;
             dots = 0;
             continue;
@@ -56,12 +77,14 @@ function normalizeStringWin32(path, allowAboveRoot) {
             res += '\\..';
           else
             res = '..';
+          lastSegmentLength = 2;
         }
       } else {
         if (res.length > 0)
           res += '\\' + path.slice(lastSlash + 1, i);
         else
           res = path.slice(lastSlash + 1, i);
+        lastSegmentLength = i - lastSlash - 1;
       }
       lastSlash = i;
       dots = 0;
@@ -77,6 +100,7 @@ function normalizeStringWin32(path, allowAboveRoot) {
 // Resolves . and .. elements in a path with directory names
 function normalizeStringPosix(path, allowAboveRoot) {
   var res = '';
+  var lastSegmentLength = 0;
   var lastSlash = -1;
   var dots = 0;
   var code;
@@ -91,27 +115,26 @@ function normalizeStringPosix(path, allowAboveRoot) {
       if (lastSlash === i - 1 || dots === 1) {
         // NOOP
       } else if (lastSlash !== i - 1 && dots === 2) {
-        if (res.length < 2 ||
+        if (res.length < 2 || lastSegmentLength !== 2 ||
             res.charCodeAt(res.length - 1) !== 46/*.*/ ||
             res.charCodeAt(res.length - 2) !== 46/*.*/) {
           if (res.length > 2) {
-            const start = res.length - 1;
-            var j = start;
-            for (; j >= 0; --j) {
-              if (res.charCodeAt(j) === 47/*/*/)
-                break;
-            }
-            if (j !== start) {
-              if (j === -1)
+            const lastSlashIndex = res.lastIndexOf('/');
+            if (lastSlashIndex !== res.length - 1) {
+              if (lastSlashIndex === -1) {
                 res = '';
-              else
-                res = res.slice(0, j);
+                lastSegmentLength = 0;
+              } else {
+                res = res.slice(0, lastSlashIndex);
+                lastSegmentLength = res.length - 1 - res.lastIndexOf('/');
+              }
               lastSlash = i;
               dots = 0;
               continue;
             }
           } else if (res.length === 2 || res.length === 1) {
             res = '';
+            lastSegmentLength = 0;
             lastSlash = i;
             dots = 0;
             continue;
@@ -122,12 +145,14 @@ function normalizeStringPosix(path, allowAboveRoot) {
             res += '/..';
           else
             res = '..';
+          lastSegmentLength = 2;
         }
       } else {
         if (res.length > 0)
           res += '/' + path.slice(lastSlash + 1, i);
         else
           res = path.slice(lastSlash + 1, i);
+        lastSegmentLength = i - lastSlash - 1;
       }
       lastSlash = i;
       dots = 0;
@@ -255,7 +280,6 @@ const win32 = {
                    (code >= 97/*a*/ && code <= 122/*z*/)) {
           // Possible device root
 
-          code = path.charCodeAt(1);
           if (path.charCodeAt(1) === 58/*:*/) {
             device = path.slice(0, 2);
             rootEnd = 2;
@@ -377,7 +401,6 @@ const win32 = {
                  (code >= 97/*a*/ && code <= 122/*z*/)) {
         // Possible device root
 
-        code = path.charCodeAt(1);
         if (path.charCodeAt(1) === 58/*:*/) {
           device = path.slice(0, 2);
           rootEnd = 2;
@@ -420,17 +443,15 @@ const win32 = {
       } else {
         return '';
       }
+    } else if (isAbsolute) {
+      if (tail.length > 0)
+        return device + '\\' + tail;
+      else
+        return device + '\\';
+    } else if (tail.length > 0) {
+      return device + tail;
     } else {
-      if (isAbsolute) {
-        if (tail.length > 0)
-          return device + '\\' + tail;
-        else
-          return device + '\\';
-      } else if (tail.length > 0) {
-        return device + tail;
-      } else {
-        return device;
-      }
+      return device;
     }
   },
 
@@ -490,7 +511,6 @@ const win32 = {
     // This means that the user can use join to construct UNC paths from
     // a server name and a share name; for example:
     //   path.join('//server', 'share') -> '\\\\server\\share\\')
-    //var firstPart = paths[0];
     var needsReplace = true;
     var slashCount = 0;
     var code = firstPart.charCodeAt(0);
@@ -756,7 +776,6 @@ const win32 = {
                  (code >= 97/*a*/ && code <= 122/*z*/)) {
         // Possible device root
 
-        code = path.charCodeAt(1);
         if (path.charCodeAt(1) === 58/*:*/) {
           rootEnd = offset = 2;
           if (len > 2) {
@@ -1024,7 +1043,6 @@ const win32 = {
                  (code >= 97/*a*/ && code <= 122/*z*/)) {
         // Possible device root
 
-        code = path.charCodeAt(1);
         if (path.charCodeAt(1) === 58/*:*/) {
           rootEnd = 2;
           if (len > 2) {
