@@ -1,3 +1,24 @@
+// Copyright Joyent, Inc. and other Node contributors.
+//
+// Permission is hereby granted, free of charge, to any person obtaining a
+// copy of this software and associated documentation files (the
+// "Software"), to deal in the Software without restriction, including
+// without limitation the rights to use, copy, modify, merge, publish,
+// distribute, sublicense, and/or sell copies of the Software, and to permit
+// persons to whom the Software is furnished to do so, subject to the
+// following conditions:
+//
+// The above copyright notice and this permission notice shall be included
+// in all copies or substantial portions of the Software.
+//
+// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
+// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
+// NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
+// DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
+// OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
+// USE OR OTHER DEALINGS IN THE SOFTWARE.
+
 // Inspiration for this code comes from Salvatore Sanfilippo's linenoise.
 // https://github.com/antirez/linenoise
 // Reference:
@@ -6,19 +27,42 @@
 
 'use strict';
 
-const kHistorySize = 30;
-
-const util = require('util');
-const debug = util.debuglog('readline');
-const internalUtil = require('internal/util');
-const inherits = util.inherits;
-const Buffer = require('buffer').Buffer;
+const { debug, inherits } = require('util');
+const { Buffer } = require('buffer');
 const EventEmitter = require('events');
+const { StringDecoder } = require('string_decoder');
+const {
+  CSI,
+  emitKeys,
+  getStringWidth,
+  isFullWidthCodePoint,
+  stripVTControlCharacters
+} = require('internal/readline');
 
+const {
+  kEscape,
+  kClearToBeginning,
+  kClearToEnd,
+  kClearLine,
+  kClearScreenDown
+} = CSI;
 
-exports.createInterface = function(input, output, completer, terminal) {
+const { now } = process.binding('timer_wrap').Timer;
+
+const kHistorySize = 30;
+const kMincrlfDelay = 100;
+// \r\n, \n, or \r followed by something other than \n
+const lineEnding = /\r?\n|\r(?!\n)/;
+
+const KEYPRESS_DECODER = Symbol('keypress-decoder');
+const ESCAPE_DECODER = Symbol('escape-decoder');
+
+// GNU readline library - keyseq-timeout is 500ms (default)
+const ESCAPE_CODE_TIMEOUT = 500;
+
+function createInterface(input, output, completer, terminal) {
   return new Interface(input, output, completer, terminal);
-};
+}
 
 
 function Interface(input, output, completer, terminal) {
@@ -26,11 +70,16 @@ function Interface(input, output, completer, terminal) {
     return new Interface(input, output, completer, terminal);
   }
 
-  this._sawReturn = false;
+  this._sawReturnAt = 0;
   this.isCompletionEnabled = true;
+  this._sawKeyPress = false;
+  this._previousKey = null;
 
   EventEmitter.call(this);
   var historySize;
+  var removeHistoryDuplicates = false;
+  let crlfDelay;
+  let prompt = '> ';
 
   if (input && input.input) {
     // an options object was given
@@ -38,18 +87,26 @@ function Interface(input, output, completer, terminal) {
     completer = input.completer;
     terminal = input.terminal;
     historySize = input.historySize;
+    removeHistoryDuplicates = input.removeHistoryDuplicates;
+    if (input.prompt !== undefined) {
+      prompt = input.prompt;
+    }
+    crlfDelay = input.crlfDelay;
     input = input.input;
   }
-  historySize = historySize || kHistorySize;
 
   if (completer && typeof completer !== 'function') {
-    throw new TypeError('Argument \'completer\' must be a function');
+    throw new TypeError('Argument "completer" must be a function');
+  }
+
+  if (historySize === undefined) {
+    historySize = kHistorySize;
   }
 
   if (typeof historySize !== 'number' ||
       isNaN(historySize) ||
       historySize < 0) {
-    throw new TypeError('Argument \'historySize\' must be a positive number');
+    throw new TypeError('Argument "historySize" must be a positive number');
   }
 
   // backwards compat; check the isTTY prop of the output stream
@@ -63,15 +120,20 @@ function Interface(input, output, completer, terminal) {
   this.output = output;
   this.input = input;
   this.historySize = historySize;
+  this.removeHistoryDuplicates = !!removeHistoryDuplicates;
+  this.crlfDelay = crlfDelay ?
+    Math.max(kMincrlfDelay, crlfDelay) : kMincrlfDelay;
 
   // Check arity, 2 - for async, 1 for sync
   if (typeof completer === 'function') {
-    this.completer = completer.length === 2 ? completer : function(v, cb) {
-      cb(null, completer(v));
-    };
+    this.completer = completer.length === 2 ?
+      completer :
+      function completerWrapper(v, cb) {
+        cb(null, completer(v));
+      };
   }
 
-  this.setPrompt('> ');
+  this.setPrompt(prompt);
 
   this.terminal = !!terminal;
 
@@ -96,6 +158,14 @@ function Interface(input, output, completer, terminal) {
 
   function onkeypress(s, key) {
     self._ttyWrite(s, key);
+    if (key && key.sequence) {
+      // if the key.sequence is half of a surrogate pair
+      // (>= 0xd800 and <= 0xdfff), refresh the line so
+      // the character is displayed appropriately.
+      const ch = key.sequence.codePointAt(0);
+      if (ch >= 0xd800 && ch <= 0xdfff)
+        self._refreshLine();
+    }
   }
 
   function onresize() {
@@ -103,18 +173,25 @@ function Interface(input, output, completer, terminal) {
   }
 
   if (!this.terminal) {
-    input.on('data', ondata);
-    input.on('end', onend);
-    self.once('close', function() {
+    function onSelfCloseWithoutTerminal() {
       input.removeListener('data', ondata);
       input.removeListener('end', onend);
-    });
-    var StringDecoder = require('string_decoder').StringDecoder; // lazy load
+    }
+
+    input.on('data', ondata);
+    input.on('end', onend);
+    self.once('close', onSelfCloseWithoutTerminal);
     this._decoder = new StringDecoder('utf8');
-
   } else {
+    function onSelfCloseWithTerminal() {
+      input.removeListener('keypress', onkeypress);
+      input.removeListener('end', ontermend);
+      if (output !== null && output !== undefined) {
+        output.removeListener('resize', onresize);
+      }
+    }
 
-    exports.emitKeypressEvents(input, this);
+    emitKeypressEvents(input, this);
 
     // input usually refers to stdin
     input.on('keypress', onkeypress);
@@ -135,13 +212,7 @@ function Interface(input, output, completer, terminal) {
     if (output !== null && output !== undefined)
       output.on('resize', onresize);
 
-    self.once('close', function() {
-      input.removeListener('keypress', onkeypress);
-      input.removeListener('end', ontermend);
-      if (output !== null && output !== undefined) {
-        output.removeListener('resize', onresize);
-      }
-    });
+    self.once('close', onSelfCloseWithTerminal);
   }
 
   input.resume();
@@ -149,11 +220,15 @@ function Interface(input, output, completer, terminal) {
 
 inherits(Interface, EventEmitter);
 
-Interface.prototype.__defineGetter__('columns', function() {
-  var columns = Infinity;
-  if (this.output && this.output.columns)
-    columns = this.output.columns;
-  return columns;
+Object.defineProperty(Interface.prototype, 'columns', {
+  configurable: true,
+  enumerable: true,
+  get: function() {
+    var columns = Infinity;
+    if (this.output && this.output.columns)
+      columns = this.output.columns;
+    return columns;
+  }
 });
 
 Interface.prototype.setPrompt = function(prompt) {
@@ -162,9 +237,13 @@ Interface.prototype.setPrompt = function(prompt) {
 
 
 Interface.prototype._setRawMode = function(mode) {
+  const wasInRawMode = this.input.isRaw;
+
   if (typeof this.input.setRawMode === 'function') {
-    return this.input.setRawMode(mode);
+    this.input.setRawMode(mode);
   }
+
+  return wasInRawMode;
 };
 
 
@@ -206,16 +285,29 @@ Interface.prototype._onLine = function(line) {
 
 Interface.prototype._writeToOutput = function _writeToOutput(stringToWrite) {
   if (typeof stringToWrite !== 'string')
-    throw new TypeError('stringToWrite must be a string');
+    throw new TypeError('"stringToWrite" argument must be a string');
 
-  if (this.output !== null && this.output !== undefined)
+  if (this.output !== null && this.output !== undefined) {
     this.output.write(stringToWrite);
+  }
 };
 
 Interface.prototype._addHistory = function() {
   if (this.line.length === 0) return '';
 
+  // if the history is disabled then return the line
+  if (this.historySize === 0) return this.line;
+
+  // if the trimmed line is empty then return the line
+  if (this.line.trim().length === 0) return this.line;
+
   if (this.history.length === 0 || this.history[0] !== this.line) {
+    if (this.removeHistoryDuplicates) {
+      // Remove older history line if identical to new one
+      const dupIndex = this.history.indexOf(this.line);
+      if (dupIndex !== -1) this.history.splice(dupIndex, 1);
+    }
+
     this.history.unshift(this.line);
 
     // Only store so many
@@ -240,13 +332,13 @@ Interface.prototype._refreshLine = function() {
   // first move to the bottom of the current line, based on cursor pos
   var prevRows = this.prevRows || 0;
   if (prevRows > 0) {
-    exports.moveCursor(this.output, 0, -prevRows);
+    moveCursor(this.output, 0, -prevRows);
   }
 
   // Cursor to left edge.
-  exports.cursorTo(this.output, 0);
+  cursorTo(this.output, 0);
   // erase data
-  exports.clearScreenDown(this.output);
+  clearScreenDown(this.output);
 
   // Write the prompt and the current buffer content.
   this._writeToOutput(line);
@@ -257,11 +349,11 @@ Interface.prototype._refreshLine = function() {
   }
 
   // Move cursor to original position.
-  exports.cursorTo(this.output, cursorPos.cols);
+  cursorTo(this.output, cursorPos.cols);
 
   var diff = lineRows - cursorPos.rows;
   if (diff > 0) {
-    exports.moveCursor(this.output, 0, -diff);
+    moveCursor(this.output, 0, -diff);
   }
 
   this.prevRows = cursorPos.rows;
@@ -302,16 +394,15 @@ Interface.prototype.write = function(d, key) {
   this.terminal ? this._ttyWrite(d, key) : this._normalWrite(d);
 };
 
-// \r\n, \n, or \r followed by something other than \n
-const lineEnding = /\r?\n|\r(?!\n)/;
 Interface.prototype._normalWrite = function(b) {
   if (b === undefined) {
     return;
   }
   var string = this._decoder.write(b);
-  if (this._sawReturn) {
+  if (this._sawReturnAt &&
+      now() - this._sawReturnAt <= this.crlfDelay) {
     string = string.replace(/^\n/, '');
-    this._sawReturn = false;
+    this._sawReturnAt = 0;
   }
 
   // Run test() on the new string chunk, not on the entire line buffer.
@@ -322,16 +413,15 @@ Interface.prototype._normalWrite = function(b) {
     this._line_buffer = null;
   }
   if (newPartContainsEnding) {
-    this._sawReturn = string.endsWith('\r');
+    this._sawReturnAt = string.endsWith('\r') ? now() : 0;
 
     // got one or more newlines; process into "line" events
     var lines = string.split(lineEnding);
-    // either '' or (concievably) the unfinished portion of the next line
+    // either '' or (conceivably) the unfinished portion of the next line
     string = lines.pop();
     this._line_buffer = string;
-    lines.forEach(function(line) {
-      this._onLine(line);
-    }, this);
+    for (var n = 0; n < lines.length; n++)
+      this._onLine(lines[n]);
   } else if (string) {
     // no newlines this time, save what we have for next time
     this._line_buffer = string;
@@ -360,11 +450,11 @@ Interface.prototype._insertString = function(c) {
   }
 };
 
-Interface.prototype._tabComplete = function() {
+Interface.prototype._tabComplete = function(lastKeypressWasTab) {
   var self = this;
 
   self.pause();
-  self.completer(self.line.slice(0, self.cursor), function(err, rv) {
+  self.completer(self.line.slice(0, self.cursor), function onComplete(err, rv) {
     self.resume();
 
     if (err) {
@@ -376,20 +466,18 @@ Interface.prototype._tabComplete = function() {
     const completeOn = rv[1];  // the text that was completed
     if (completions && completions.length) {
       // Apply/show completions.
-      if (completions.length === 1) {
-        self._insertString(completions[0].slice(completeOn.length));
-      } else {
+      if (lastKeypressWasTab) {
         self._writeToOutput('\r\n');
-        var width = completions.reduce(function(a, b) {
+        var width = completions.reduce(function completionReducer(a, b) {
           return a.length > b.length ? a : b;
         }).length + 2;  // 2 space padding
         var maxColumns = Math.floor(self.columns / width);
         if (!maxColumns || maxColumns === Infinity) {
           maxColumns = 1;
         }
-        var group = [], c;
-        for (var i = 0, compLen = completions.length; i < compLen; i++) {
-          c = completions[i];
+        var group = [];
+        for (var i = 0; i < completions.length; i++) {
+          var c = completions[i];
           if (c === '') {
             handleGroup(self, group, width, maxColumns);
             group = [];
@@ -398,16 +486,17 @@ Interface.prototype._tabComplete = function() {
           }
         }
         handleGroup(self, group, width, maxColumns);
-
-        // If there is a common prefix to all matches, then apply that
-        // portion.
-        var f = completions.filter(function(e) { if (e) return e; });
-        var prefix = commonPrefix(f);
-        if (prefix.length > completeOn.length) {
-          self._insertString(prefix.slice(completeOn.length));
-        }
-
       }
+
+      // If there is a common prefix to all matches, then apply that portion.
+      var f = completions.filter(function completionFilter(e) {
+        if (e) return e;
+      });
+      var prefix = commonPrefix(f);
+      if (prefix.length > completeOn.length) {
+        self._insertString(prefix.slice(completeOn.length));
+      }
+
       self._refreshLine();
     }
   });
@@ -428,8 +517,7 @@ function handleGroup(self, group, width, maxColumns) {
       var item = group[idx];
       self._writeToOutput(item);
       if (col < maxColumns - 1) {
-        for (var s = 0, itemLen = item.length; s < width - itemLen;
-             s++) {
+        for (var s = 0; s < width - item.length; s++) {
           self._writeToOutput(' ');
         }
       }
@@ -443,6 +531,7 @@ function commonPrefix(strings) {
   if (!strings || strings.length === 0) {
     return '';
   }
+  if (strings.length === 1) return strings[0];
   var sorted = strings.slice().sort();
   var min = sorted[0];
   var max = sorted[sorted.length - 1];
@@ -458,7 +547,7 @@ function commonPrefix(strings) {
 Interface.prototype._wordLeft = function() {
   if (this.cursor > 0) {
     var leading = this.line.slice(0, this.cursor);
-    var match = leading.match(/([^\w\s]+|\w+|)\s*$/);
+    var match = leading.match(/(?:[^\w\s]+|\w+|)\s*$/);
     this._moveCursor(-match[0].length);
   }
 };
@@ -467,7 +556,7 @@ Interface.prototype._wordLeft = function() {
 Interface.prototype._wordRight = function() {
   if (this.cursor < this.line.length) {
     var trailing = this.line.slice(this.cursor);
-    var match = trailing.match(/^(\s+|\W+|\w+)\s*/);
+    var match = trailing.match(/^(?:\s+|\W+|\w+)\s*/);
     this._moveCursor(match[0].length);
   }
 };
@@ -494,7 +583,7 @@ Interface.prototype._deleteRight = function() {
 Interface.prototype._deleteWordLeft = function() {
   if (this.cursor > 0) {
     var leading = this.line.slice(0, this.cursor);
-    var match = leading.match(/([^\w\s]+|\w+|)\s*$/);
+    var match = leading.match(/(?:[^\w\s]+|\w+|)\s*$/);
     leading = leading.slice(0, leading.length - match[0].length);
     this.line = leading + this.line.slice(this.cursor, this.line.length);
     this.cursor = leading.length;
@@ -506,7 +595,7 @@ Interface.prototype._deleteWordLeft = function() {
 Interface.prototype._deleteWordRight = function() {
   if (this.cursor < this.line.length) {
     var trailing = this.line.slice(this.cursor);
-    var match = trailing.match(/^(\s+|\W+|\w+)\s*/);
+    var match = trailing.match(/^(?:\s+|\W+|\w+)\s*/);
     this.line = this.line.slice(0, this.cursor) +
                 trailing.slice(match[0].length);
     this._refreshLine();
@@ -587,18 +676,19 @@ Interface.prototype._getDisplayPos = function(str) {
       row += 1;
       continue;
     }
-    if (isFullWidthCodePoint(code)) {
+    const width = getStringWidth(code);
+    if (width === 0 || width === 1) {
+      offset += width;
+    } else { // width === 2
       if ((offset + 1) % col === 0) {
         offset++;
       }
       offset += 2;
-    } else {
-      offset++;
     }
   }
   var cols = offset % col;
   var rows = row + (offset - cols) / col;
-  return {cols: cols, rows: rows};
+  return { cols: cols, rows: rows };
 };
 
 
@@ -617,7 +707,7 @@ Interface.prototype._getCursorPos = function() {
     rows++;
     cols = 0;
   }
-  return {cols: cols, rows: rows};
+  return { cols: cols, rows: rows };
 };
 
 
@@ -640,14 +730,14 @@ Interface.prototype._moveCursor = function(dx) {
     var diffWidth;
     if (diffCursor < 0) {
       diffWidth = -getStringWidth(
-          this.line.substring(this.cursor, oldcursor)
-          );
+        this.line.substring(this.cursor, oldcursor)
+      );
     } else if (diffCursor > 0) {
       diffWidth = getStringWidth(
-          this.line.substring(this.cursor, oldcursor)
-          );
+        this.line.substring(this.cursor, oldcursor)
+      );
     }
-    exports.moveCursor(this.output, diffWidth, 0);
+    moveCursor(this.output, diffWidth, 0);
     this.prevRows = newPos.rows;
   } else {
     this._refreshLine();
@@ -657,7 +747,9 @@ Interface.prototype._moveCursor = function(dx) {
 
 // handle a write from the tty
 Interface.prototype._ttyWrite = function(s, key) {
+  const previousKey = this._previousKey;
   key = key || {};
+  this._previousKey = key;
 
   // Ignore escape key - Fixes #2876
   if (key.name === 'escape') return;
@@ -727,8 +819,8 @@ Interface.prototype._ttyWrite = function(s, key) {
         break;
 
       case 'l': // clear the whole screen
-        exports.cursorTo(this.output, 0, 0);
-        exports.clearScreenDown(this.output);
+        cursorTo(this.output, 0, 0);
+        clearScreenDown(this.output);
         this._refreshLine();
         break;
 
@@ -810,20 +902,22 @@ Interface.prototype._ttyWrite = function(s, key) {
     /* No modifier keys used */
 
     // \r bookkeeping is only relevant if a \n comes right after.
-    if (this._sawReturn && key.name !== 'enter')
-      this._sawReturn = false;
+    if (this._sawReturnAt && key.name !== 'enter')
+      this._sawReturnAt = 0;
 
     switch (key.name) {
       case 'return':  // carriage return, i.e. \r
-        this._sawReturn = true;
+        this._sawReturnAt = now();
         this._line();
         break;
 
       case 'enter':
-        if (this._sawReturn)
-          this._sawReturn = false;
-        else
+        // When key interval > crlfDelay
+        if (this._sawReturnAt === 0 ||
+            now() - this._sawReturnAt > this.crlfDelay) {
           this._line();
+        }
+        this._sawReturnAt = 0;
         break;
 
       case 'backspace':
@@ -861,7 +955,8 @@ Interface.prototype._ttyWrite = function(s, key) {
       case 'tab':
         // If tab completion enabled, do that...
         if (typeof this.completer === 'function' && this.isCompletionEnabled) {
-          this._tabComplete();
+          const lastKeypressWasTab = previousKey && previousKey.name === 'tab';
+          this._tabComplete(lastKeypressWasTab);
           break;
         }
         // falls through
@@ -883,23 +978,12 @@ Interface.prototype._ttyWrite = function(s, key) {
   }
 };
 
-
-exports.Interface = Interface;
-
-
 /**
  * accepts a readable Stream instance and makes it emit "keypress" events
  */
 
-const KEYPRESS_DECODER = Symbol('keypress-decoder');
-const ESCAPE_DECODER = Symbol('escape-decoder');
-
-// GNU readline library - keyseq-timeout is 500ms (default)
-const ESCAPE_CODE_TIMEOUT = 500;
-
 function emitKeypressEvents(stream, iface) {
   if (stream[KEYPRESS_DECODER]) return;
-  var StringDecoder = require('string_decoder').StringDecoder; // lazy load
   stream[KEYPRESS_DECODER] = new StringDecoder('utf8');
 
   stream[ESCAPE_DECODER] = emitKeys(stream);
@@ -914,6 +998,10 @@ function emitKeypressEvents(stream, iface) {
       if (r) {
         clearTimeout(timeoutId);
 
+        if (iface) {
+          iface._sawKeyPress = r.length === 1;
+        }
+
         for (var i = 0; i < r.length; i++) {
           if (r[i] === '\t' && typeof r[i + 1] === 'string' && iface) {
             iface.isCompletionEnabled = false;
@@ -922,7 +1010,7 @@ function emitKeypressEvents(stream, iface) {
           try {
             stream[ESCAPE_DECODER].next(r[i]);
             // Escape letter at the tail position
-            if (r[i] === '\x1b' && i + 1 === r.length) {
+            if (r[i] === kEscape && i + 1 === r.length) {
               timeoutId = setTimeout(escapeCodeTimeout, ESCAPE_CODE_TIMEOUT);
             }
           } catch (err) {
@@ -958,323 +1046,6 @@ function emitKeypressEvents(stream, iface) {
     stream.on('newListener', onNewListener);
   }
 }
-exports.emitKeypressEvents = emitKeypressEvents;
-
-/*
-  Some patterns seen in terminal key escape codes, derived from combos seen
-  at http://www.midnight-commander.org/browser/lib/tty/key.c
-
-  ESC letter
-  ESC [ letter
-  ESC [ modifier letter
-  ESC [ 1 ; modifier letter
-  ESC [ num char
-  ESC [ num ; modifier char
-  ESC O letter
-  ESC O modifier letter
-  ESC O 1 ; modifier letter
-  ESC N letter
-  ESC [ [ num ; modifier char
-  ESC [ [ 1 ; modifier letter
-  ESC ESC [ num char
-  ESC ESC O letter
-
-  - char is usually ~ but $ and ^ also happen with rxvt
-  - modifier is 1 +
-                (shift     * 1) +
-                (left_alt  * 2) +
-                (ctrl      * 4) +
-                (right_alt * 8)
-  - two leading ESCs apparently mean the same as one leading ESC
-*/
-
-// Regexes used for ansi escape code splitting
-// eslint-disable-next-line no-control-regex
-const metaKeyCodeReAnywhere = /(?:\x1b)([a-zA-Z0-9])/;
-const functionKeyCodeReAnywhere = new RegExp('(?:\x1b+)(O|N|\\[|\\[\\[)(?:' + [
-  '(\\d+)(?:;(\\d+))?([~^$])',
-  '(?:M([@ #!a`])(.)(.))', // mouse
-  '(?:1;)?(\\d+)?([a-zA-Z])'
-].join('|') + ')');
-
-
-function* emitKeys(stream) {
-  while (true) {
-    var ch = yield;
-    var s = ch;
-    var escaped = false;
-    var key = {
-      sequence: null,
-      name: undefined,
-      ctrl: false,
-      meta: false,
-      shift: false
-    };
-
-    if (ch === '\x1b') {
-      escaped = true;
-      s += (ch = yield);
-
-      if (ch === '\x1b') {
-        s += (ch = yield);
-      }
-    }
-
-    if (escaped && (ch === 'O' || ch === '[')) {
-      // ansi escape sequence
-      var code = ch;
-      var modifier = 0;
-
-      if (ch === 'O') {
-        // ESC O letter
-        // ESC O modifier letter
-        s += (ch = yield);
-
-        if (ch >= '0' && ch <= '9') {
-          modifier = (ch >> 0) - 1;
-          s += (ch = yield);
-        }
-
-        code += ch;
-
-      } else if (ch === '[') {
-        // ESC [ letter
-        // ESC [ modifier letter
-        // ESC [ [ modifier letter
-        // ESC [ [ num char
-        s += (ch = yield);
-
-        if (ch === '[') {
-          // \x1b[[A
-          //      ^--- escape codes might have a second bracket
-          code += ch;
-          s += (ch = yield);
-        }
-
-        /*
-         * Here and later we try to buffer just enough data to get
-         * a complete ascii sequence.
-         *
-         * We have basically two classes of ascii characters to process:
-         *
-         *
-         * 1. `\x1b[24;5~` should be parsed as { code: '[24~', modifier: 5 }
-         *
-         * This particular example is featuring Ctrl+F12 in xterm.
-         *
-         *  - `;5` part is optional, e.g. it could be `\x1b[24~`
-         *  - first part can contain one or two digits
-         *
-         * So the generic regexp is like /^\d\d?(;\d)?[~^$]$/
-         *
-         *
-         * 2. `\x1b[1;5H` should be parsed as { code: '[H', modifier: 5 }
-         *
-         * This particular example is featuring Ctrl+Home in xterm.
-         *
-         *  - `1;5` part is optional, e.g. it could be `\x1b[H`
-         *  - `1;` part is optional, e.g. it could be `\x1b[5H`
-         *
-         * So the generic regexp is like /^((\d;)?\d)?[A-Za-z]$/
-         *
-         */
-        const cmdStart = s.length - 1;
-
-        // skip one or two leading digits
-        if (ch >= '0' && ch <= '9') {
-          s += (ch = yield);
-
-          if (ch >= '0' && ch <= '9') {
-            s += (ch = yield);
-          }
-        }
-
-        // skip modifier
-        if (ch === ';') {
-          s += (ch = yield);
-
-          if (ch >= '0' && ch <= '9') {
-            s += (ch = yield);
-          }
-        }
-
-        /*
-         * We buffered enough data, now trying to extract code
-         * and modifier from it
-         */
-        const cmd = s.slice(cmdStart);
-        var match;
-
-        if ((match = cmd.match(/^(\d\d?)(;(\d))?([~^$])$/))) {
-          code += match[1] + match[4];
-          modifier = (match[3] || 1) - 1;
-        } else if ((match = cmd.match(/^((\d;)?(\d))?([A-Za-z])$/))) {
-          code += match[4];
-          modifier = (match[3] || 1) - 1;
-        } else {
-          code += cmd;
-        }
-      }
-
-      // Parse the key modifier
-      key.ctrl = !!(modifier & 4);
-      key.meta = !!(modifier & 10);
-      key.shift = !!(modifier & 1);
-      key.code = code;
-
-      // Parse the key itself
-      switch (code) {
-        /* xterm/gnome ESC O letter */
-        case 'OP': key.name = 'f1'; break;
-        case 'OQ': key.name = 'f2'; break;
-        case 'OR': key.name = 'f3'; break;
-        case 'OS': key.name = 'f4'; break;
-
-        /* xterm/rxvt ESC [ number ~ */
-        case '[11~': key.name = 'f1'; break;
-        case '[12~': key.name = 'f2'; break;
-        case '[13~': key.name = 'f3'; break;
-        case '[14~': key.name = 'f4'; break;
-
-        /* from Cygwin and used in libuv */
-        case '[[A': key.name = 'f1'; break;
-        case '[[B': key.name = 'f2'; break;
-        case '[[C': key.name = 'f3'; break;
-        case '[[D': key.name = 'f4'; break;
-        case '[[E': key.name = 'f5'; break;
-
-        /* common */
-        case '[15~': key.name = 'f5'; break;
-        case '[17~': key.name = 'f6'; break;
-        case '[18~': key.name = 'f7'; break;
-        case '[19~': key.name = 'f8'; break;
-        case '[20~': key.name = 'f9'; break;
-        case '[21~': key.name = 'f10'; break;
-        case '[23~': key.name = 'f11'; break;
-        case '[24~': key.name = 'f12'; break;
-
-        /* xterm ESC [ letter */
-        case '[A': key.name = 'up'; break;
-        case '[B': key.name = 'down'; break;
-        case '[C': key.name = 'right'; break;
-        case '[D': key.name = 'left'; break;
-        case '[E': key.name = 'clear'; break;
-        case '[F': key.name = 'end'; break;
-        case '[H': key.name = 'home'; break;
-
-        /* xterm/gnome ESC O letter */
-        case 'OA': key.name = 'up'; break;
-        case 'OB': key.name = 'down'; break;
-        case 'OC': key.name = 'right'; break;
-        case 'OD': key.name = 'left'; break;
-        case 'OE': key.name = 'clear'; break;
-        case 'OF': key.name = 'end'; break;
-        case 'OH': key.name = 'home'; break;
-
-        /* xterm/rxvt ESC [ number ~ */
-        case '[1~': key.name = 'home'; break;
-        case '[2~': key.name = 'insert'; break;
-        case '[3~': key.name = 'delete'; break;
-        case '[4~': key.name = 'end'; break;
-        case '[5~': key.name = 'pageup'; break;
-        case '[6~': key.name = 'pagedown'; break;
-
-        /* putty */
-        case '[[5~': key.name = 'pageup'; break;
-        case '[[6~': key.name = 'pagedown'; break;
-
-        /* rxvt */
-        case '[7~': key.name = 'home'; break;
-        case '[8~': key.name = 'end'; break;
-
-        /* rxvt keys with modifiers */
-        case '[a': key.name = 'up'; key.shift = true; break;
-        case '[b': key.name = 'down'; key.shift = true; break;
-        case '[c': key.name = 'right'; key.shift = true; break;
-        case '[d': key.name = 'left'; key.shift = true; break;
-        case '[e': key.name = 'clear'; key.shift = true; break;
-
-        case '[2$': key.name = 'insert'; key.shift = true; break;
-        case '[3$': key.name = 'delete'; key.shift = true; break;
-        case '[5$': key.name = 'pageup'; key.shift = true; break;
-        case '[6$': key.name = 'pagedown'; key.shift = true; break;
-        case '[7$': key.name = 'home'; key.shift = true; break;
-        case '[8$': key.name = 'end'; key.shift = true; break;
-
-        case 'Oa': key.name = 'up'; key.ctrl = true; break;
-        case 'Ob': key.name = 'down'; key.ctrl = true; break;
-        case 'Oc': key.name = 'right'; key.ctrl = true; break;
-        case 'Od': key.name = 'left'; key.ctrl = true; break;
-        case 'Oe': key.name = 'clear'; key.ctrl = true; break;
-
-        case '[2^': key.name = 'insert'; key.ctrl = true; break;
-        case '[3^': key.name = 'delete'; key.ctrl = true; break;
-        case '[5^': key.name = 'pageup'; key.ctrl = true; break;
-        case '[6^': key.name = 'pagedown'; key.ctrl = true; break;
-        case '[7^': key.name = 'home'; key.ctrl = true; break;
-        case '[8^': key.name = 'end'; key.ctrl = true; break;
-
-        /* misc. */
-        case '[Z': key.name = 'tab'; key.shift = true; break;
-        default: key.name = 'undefined'; break;
-      }
-
-    } else if (ch === '\r') {
-      // carriage return
-      key.name = 'return';
-
-    } else if (ch === '\n') {
-      // enter, should have been called linefeed
-      key.name = 'enter';
-
-    } else if (ch === '\t') {
-      // tab
-      key.name = 'tab';
-
-    } else if (ch === '\b' || ch === '\x7f') {
-      // backspace or ctrl+h
-      key.name = 'backspace';
-      key.meta = escaped;
-
-    } else if (ch === '\x1b') {
-      // escape key
-      key.name = 'escape';
-      key.meta = escaped;
-
-    } else if (ch === ' ') {
-      key.name = 'space';
-      key.meta = escaped;
-
-    } else if (!escaped && ch <= '\x1a') {
-      // ctrl+letter
-      key.name = String.fromCharCode(ch.charCodeAt(0) + 'a'.charCodeAt(0) - 1);
-      key.ctrl = true;
-
-    } else if (/^[0-9A-Za-z]$/.test(ch)) {
-      // letter, number, shift+letter
-      key.name = ch.toLowerCase();
-      key.shift = /^[A-Z]$/.test(ch);
-      key.meta = escaped;
-    } else if (escaped) {
-      // Escape sequence timeout
-      key.name = ch.length ? undefined : 'escape';
-      key.meta = true;
-    }
-
-    key.sequence = s;
-
-    if (s.length !== 0 && (key.name !== undefined || escaped)) {
-      /* Named character or sequence */
-      stream.emit('keypress', escaped ? undefined : s, key);
-    } else if (s.length === 1) {
-      /* Single unnamed character, e.g. "." */
-      stream.emit('keypress', s);
-    } else {
-      /* Unrecognized or broken escape sequence, don't emit anything */
-    }
-  }
-}
-
 
 /**
  * moves the cursor to the x and y coordinate on the given stream
@@ -1288,16 +1059,14 @@ function cursorTo(stream, x, y) {
     return;
 
   if (typeof x !== 'number')
-    throw new Error("Can't set cursor row without also setting it's column");
+    throw new Error('Can\'t set cursor row without also setting it\'s column');
 
   if (typeof y !== 'number') {
-    stream.write('\x1b[' + (x + 1) + 'G');
+    stream.write(CSI`${x + 1}G`);
   } else {
-    stream.write('\x1b[' + (y + 1) + ';' + (x + 1) + 'H');
+    stream.write(CSI`${y + 1};${x + 1}H`);
   }
 }
-exports.cursorTo = cursorTo;
-
 
 /**
  * moves the cursor relative to its current location
@@ -1308,19 +1077,17 @@ function moveCursor(stream, dx, dy) {
     return;
 
   if (dx < 0) {
-    stream.write('\x1b[' + (-dx) + 'D');
+    stream.write(CSI`${-dx}D`);
   } else if (dx > 0) {
-    stream.write('\x1b[' + dx + 'C');
+    stream.write(CSI`${dx}C`);
   }
 
   if (dy < 0) {
-    stream.write('\x1b[' + (-dy) + 'A');
+    stream.write(CSI`${-dy}A`);
   } else if (dy > 0) {
-    stream.write('\x1b[' + dy + 'B');
+    stream.write(CSI`${dy}B`);
   }
 }
-exports.moveCursor = moveCursor;
-
 
 /**
  * clears the current line the cursor is on:
@@ -1335,17 +1102,15 @@ function clearLine(stream, dir) {
 
   if (dir < 0) {
     // to the beginning
-    stream.write('\x1b[1K');
+    stream.write(kClearToBeginning);
   } else if (dir > 0) {
     // to the end
-    stream.write('\x1b[0K');
+    stream.write(kClearToEnd);
   } else {
     // entire line
-    stream.write('\x1b[2K');
+    stream.write(kClearLine);
   }
 }
-exports.clearLine = clearLine;
-
 
 /**
  * clears the screen from the current position of the cursor down
@@ -1355,110 +1120,15 @@ function clearScreenDown(stream) {
   if (stream === null || stream === undefined)
     return;
 
-  stream.write('\x1b[0J');
+  stream.write(kClearScreenDown);
 }
-exports.clearScreenDown = clearScreenDown;
 
-
-/**
- * Returns the number of columns required to display the given string.
- */
-
-function getStringWidth(str) {
-  var width = 0;
-  str = stripVTControlCharacters(str);
-  for (var i = 0, len = str.length; i < len; i++) {
-    var code = str.codePointAt(i);
-    if (code >= 0x10000) { // surrogates
-      i++;
-    }
-    if (isFullWidthCodePoint(code)) {
-      width += 2;
-    } else {
-      width++;
-    }
-  }
-  return width;
-}
-exports.getStringWidth = getStringWidth;
-
-
-/**
- * Returns true if the character represented by a given
- * Unicode code point is full-width. Otherwise returns false.
- */
-
-function isFullWidthCodePoint(code) {
-  if (isNaN(code)) {
-    return false;
-  }
-
-  // Code points are derived from:
-  // http://www.unicode.org/Public/UNIDATA/EastAsianWidth.txt
-  if (code >= 0x1100 && (
-      code <= 0x115f ||  // Hangul Jamo
-      0x2329 === code || // LEFT-POINTING ANGLE BRACKET
-      0x232a === code || // RIGHT-POINTING ANGLE BRACKET
-      // CJK Radicals Supplement .. Enclosed CJK Letters and Months
-      (0x2e80 <= code && code <= 0x3247 && code !== 0x303f) ||
-      // Enclosed CJK Letters and Months .. CJK Unified Ideographs Extension A
-      0x3250 <= code && code <= 0x4dbf ||
-      // CJK Unified Ideographs .. Yi Radicals
-      0x4e00 <= code && code <= 0xa4c6 ||
-      // Hangul Jamo Extended-A
-      0xa960 <= code && code <= 0xa97c ||
-      // Hangul Syllables
-      0xac00 <= code && code <= 0xd7a3 ||
-      // CJK Compatibility Ideographs
-      0xf900 <= code && code <= 0xfaff ||
-      // Vertical Forms
-      0xfe10 <= code && code <= 0xfe19 ||
-      // CJK Compatibility Forms .. Small Form Variants
-      0xfe30 <= code && code <= 0xfe6b ||
-      // Halfwidth and Fullwidth Forms
-      0xff01 <= code && code <= 0xff60 ||
-      0xffe0 <= code && code <= 0xffe6 ||
-      // Kana Supplement
-      0x1b000 <= code && code <= 0x1b001 ||
-      // Enclosed Ideographic Supplement
-      0x1f200 <= code && code <= 0x1f251 ||
-      // CJK Unified Ideographs Extension B .. Tertiary Ideographic Plane
-      0x20000 <= code && code <= 0x3fffd)) {
-    return true;
-  }
-  return false;
-}
-exports.isFullWidthCodePoint = isFullWidthCodePoint;
-
-
-/**
- * Returns the Unicode code point for the character at the
- * given index in the given string. Similar to String.charCodeAt(),
- * but this function handles surrogates (code point >= 0x10000).
- */
-
-function codePointAt(str, index) {
-  var code = str.charCodeAt(index);
-  var low;
-  if (0xd800 <= code && code <= 0xdbff) { // High surrogate
-    low = str.charCodeAt(index + 1);
-    if (!isNaN(low)) {
-      code = 0x10000 + (code - 0xd800) * 0x400 + (low - 0xdc00);
-    }
-  }
-  return code;
-}
-exports.codePointAt = internalUtil.deprecate(codePointAt,
-    'readline.codePointAt is deprecated. ' +
-    'Use String.prototype.codePointAt instead.');
-
-
-/**
- * Tries to remove all VT control characters. Use to estimate displayed
- * string width. May be buggy due to not running a real state machine
- */
-function stripVTControlCharacters(str) {
-  str = str.replace(new RegExp(functionKeyCodeReAnywhere.source, 'g'), '');
-  return str.replace(new RegExp(metaKeyCodeReAnywhere.source, 'g'), '');
-}
-exports.stripVTControlCharacters = stripVTControlCharacters;
+module.exports = {
+  Interface,
+  clearLine,
+  clearScreenDown,
+  createInterface,
+  cursorTo,
+  emitKeypressEvents,
+  moveCursor
+};
