@@ -1,7 +1,34 @@
+// Copyright Joyent, Inc. and other Node contributors.
+//
+// Permission is hereby granted, free of charge, to any person obtaining a
+// copy of this software and associated documentation files (the
+// "Software"), to deal in the Software without restriction, including
+// without limitation the rights to use, copy, modify, merge, publish,
+// distribute, sublicense, and/or sell copies of the Software, and to permit
+// persons to whom the Software is furnished to do so, subject to the
+// following conditions:
+//
+// The above copyright notice and this permission notice shall be included
+// in all copies or substantial portions of the Software.
+//
+// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
+// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
+// NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
+// DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
+// OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
+// USE OR OTHER DEALINGS IN THE SOFTWARE.
+
 'use strict';
 
-const binding = process.binding('contextify');
-const Script = binding.ContextifyScript;
+const {
+  ContextifyScript: Script,
+  kParsingContext,
+
+  makeContext,
+  isContext,
+  runInDebugContext
+} = process.binding('contextify');
 
 // The binding provides a few useful primitives:
 // - Script(code, { filename = "evalmachine.anonymous",
@@ -17,20 +44,17 @@ const realRunInThisContext = Script.prototype.runInThisContext;
 const realRunInContext = Script.prototype.runInContext;
 
 Script.prototype.runInThisContext = function(options) {
-  if (options && options.breakOnSigint) {
-    return sigintHandlersWrap(() => {
-      return realRunInThisContext.call(this, options);
-    });
+  if (options && options.breakOnSigint && process._events.SIGINT) {
+    return sigintHandlersWrap(realRunInThisContext, this, [options]);
   } else {
     return realRunInThisContext.call(this, options);
   }
 };
 
 Script.prototype.runInContext = function(contextifiedSandbox, options) {
-  if (options && options.breakOnSigint) {
-    return sigintHandlersWrap(() => {
-      return realRunInContext.call(this, contextifiedSandbox, options);
-    });
+  if (options && options.breakOnSigint && process._events.SIGINT) {
+    return sigintHandlersWrap(realRunInContext, this,
+                              [contextifiedSandbox, options]);
   } else {
     return realRunInContext.call(this, contextifiedSandbox, options);
   }
@@ -44,11 +68,11 @@ Script.prototype.runInNewContext = function(sandbox, options) {
 function createContext(sandbox) {
   if (sandbox === undefined) {
     sandbox = {};
-  } else if (binding.isContext(sandbox)) {
+  } else if (isContext(sandbox)) {
     return sandbox;
   }
 
-  binding.makeContext(sandbox);
+  makeContext(sandbox);
   return sandbox;
 }
 
@@ -58,19 +82,20 @@ function createScript(code, options) {
 
 // Remove all SIGINT listeners and re-attach them after the wrapped function
 // has executed, so that caught SIGINT are handled by the listeners again.
-function sigintHandlersWrap(fn) {
+function sigintHandlersWrap(fn, thisArg, argsArray) {
   // Using the internal list here to make sure `.once()` wrappers are used,
   // not the original ones.
   let sigintListeners = process._events.SIGINT;
-  if (!Array.isArray(sigintListeners))
-    sigintListeners = sigintListeners ? [sigintListeners] : [];
-  else
+
+  if (Array.isArray(sigintListeners))
     sigintListeners = sigintListeners.slice();
+  else
+    sigintListeners = [sigintListeners];
 
   process.removeAllListeners('SIGINT');
 
   try {
-    return fn();
+    return fn.apply(thisArg, argsArray);
   } finally {
     // Add using the public methods so that the `newListener` handler of
     // process can re-attach the listeners.
@@ -80,16 +105,33 @@ function sigintHandlersWrap(fn) {
   }
 }
 
-function runInDebugContext(code) {
-  return binding.runInDebugContext(code);
-}
-
 function runInContext(code, contextifiedSandbox, options) {
+  if (typeof options === 'string') {
+    options = {
+      filename: options,
+      [kParsingContext]: contextifiedSandbox
+    };
+  } else {
+    options = Object.assign({}, options, {
+      [kParsingContext]: contextifiedSandbox
+    });
+  }
   return createScript(code, options)
     .runInContext(contextifiedSandbox, options);
 }
 
 function runInNewContext(code, sandbox, options) {
+  sandbox = createContext(sandbox);
+  if (typeof options === 'string') {
+    options = {
+      filename: options,
+      [kParsingContext]: sandbox
+    };
+  } else {
+    options = Object.assign({}, options, {
+      [kParsingContext]: sandbox
+    });
+  }
   return createScript(code, options).runInNewContext(sandbox, options);
 }
 
@@ -105,5 +147,5 @@ module.exports = {
   runInContext,
   runInNewContext,
   runInThisContext,
-  isContext: binding.isContext
+  isContext
 };
