@@ -1,7 +1,25 @@
-#include "node.h"
-#include "v8.h"
-#include "env.h"
-#include "env-inl.h"
+// Copyright Joyent, Inc. and other Node contributors.
+//
+// Permission is hereby granted, free of charge, to any person obtaining a
+// copy of this software and associated documentation files (the
+// "Software"), to deal in the Software without restriction, including
+// without limitation the rights to use, copy, modify, merge, publish,
+// distribute, sublicense, and/or sell copies of the Software, and to permit
+// persons to whom the Software is furnished to do so, subject to the
+// following conditions:
+//
+// The above copyright notice and this permission notice shall be included
+// in all copies or substantial portions of the Software.
+//
+// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
+// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
+// NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
+// DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
+// OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
+// USE OR OTHER DEALINGS IN THE SOFTWARE.
+
+#include "node_internals.h"
 #include "string_bytes.h"
 
 #include <errno.h>
@@ -32,6 +50,7 @@ using v8::ArrayBuffer;
 using v8::Boolean;
 using v8::Context;
 using v8::Float64Array;
+using v8::Function;
 using v8::FunctionCallbackInfo;
 using v8::Integer;
 using v8::Local;
@@ -123,36 +142,47 @@ static void GetOSRelease(const FunctionCallbackInfo<Value>& args) {
 static void GetCPUInfo(const FunctionCallbackInfo<Value>& args) {
   Environment* env = Environment::GetCurrent(args);
   uv_cpu_info_t* cpu_infos;
-  int count, i;
+  int count, i, field_idx;
 
   int err = uv_cpu_info(&cpu_infos, &count);
   if (err)
     return;
 
-  Local<Array> cpus = Array::New(env->isolate());
-  for (i = 0; i < count; i++) {
+  CHECK(args[0]->IsFunction());
+  Local<Function> addfn = args[0].As<Function>();
+
+  CHECK(args[1]->IsFloat64Array());
+  Local<Float64Array> array = args[1].As<Float64Array>();
+  CHECK_EQ(array->Length(), 6 * NODE_PUSH_VAL_TO_ARRAY_MAX);
+  Local<ArrayBuffer> ab = array->Buffer();
+  double* fields = static_cast<double*>(ab->GetContents().Data());
+
+  CHECK(args[2]->IsArray());
+  Local<Array> cpus = args[2].As<Array>();
+
+  Local<Value> model_argv[NODE_PUSH_VAL_TO_ARRAY_MAX];
+  int model_idx = 0;
+
+  for (i = 0, field_idx = 0; i < count; i++) {
     uv_cpu_info_t* ci = cpu_infos + i;
 
-    Local<Object> times_info = Object::New(env->isolate());
-    times_info->Set(env->user_string(),
-                    Number::New(env->isolate(), ci->cpu_times.user));
-    times_info->Set(env->nice_string(),
-                    Number::New(env->isolate(), ci->cpu_times.nice));
-    times_info->Set(env->sys_string(),
-                    Number::New(env->isolate(), ci->cpu_times.sys));
-    times_info->Set(env->idle_string(),
-                    Number::New(env->isolate(), ci->cpu_times.idle));
-    times_info->Set(env->irq_string(),
-                    Number::New(env->isolate(), ci->cpu_times.irq));
+    fields[field_idx++] = ci->speed;
+    fields[field_idx++] = ci->cpu_times.user;
+    fields[field_idx++] = ci->cpu_times.nice;
+    fields[field_idx++] = ci->cpu_times.sys;
+    fields[field_idx++] = ci->cpu_times.idle;
+    fields[field_idx++] = ci->cpu_times.irq;
+    model_argv[model_idx++] = OneByteString(env->isolate(), ci->model);
 
-    Local<Object> cpu_info = Object::New(env->isolate());
-    cpu_info->Set(env->model_string(),
-                  OneByteString(env->isolate(), ci->model));
-    cpu_info->Set(env->speed_string(),
-                  Number::New(env->isolate(), ci->speed));
-    cpu_info->Set(env->times_string(), times_info);
+    if (model_idx >= NODE_PUSH_VAL_TO_ARRAY_MAX) {
+      addfn->Call(env->context(), cpus, model_idx, model_argv).ToLocalChecked();
+      model_idx = 0;
+      field_idx = 0;
+    }
+  }
 
-    (*cpus)->Set(i, cpu_info);
+  if (model_idx > 0) {
+    addfn->Call(env->context(), cpus, model_idx, model_argv).ToLocalChecked();
   }
 
   uv_free_cpu_info(cpu_infos, count);
@@ -324,36 +354,43 @@ static void GetUserInfo(const FunctionCallbackInfo<Value>& args) {
     return env->ThrowUVException(err, "uv_os_get_passwd");
   }
 
+  Local<Value> error;
+
   Local<Value> uid = Number::New(env->isolate(), pwd.uid);
   Local<Value> gid = Number::New(env->isolate(), pwd.gid);
-  Local<Value> username = StringBytes::Encode(env->isolate(),
-                                              pwd.username,
-                                              encoding);
-  Local<Value> homedir = StringBytes::Encode(env->isolate(),
-                                             pwd.homedir,
-                                             encoding);
-  Local<Value> shell;
+  MaybeLocal<Value> username = StringBytes::Encode(env->isolate(),
+                                                   pwd.username,
+                                                   encoding,
+                                                   &error);
+  MaybeLocal<Value> homedir = StringBytes::Encode(env->isolate(),
+                                                  pwd.homedir,
+                                                  encoding,
+                                                  &error);
+  MaybeLocal<Value> shell;
 
-  if (pwd.shell == NULL)
+  if (pwd.shell == nullptr)
     shell = Null(env->isolate());
   else
-    shell = StringBytes::Encode(env->isolate(), pwd.shell, encoding);
+    shell = StringBytes::Encode(env->isolate(), pwd.shell, encoding, &error);
 
   uv_os_free_passwd(&pwd);
 
   if (username.IsEmpty()) {
+    // TODO(addaleax): Use `error` itself here.
     return env->ThrowUVException(UV_EINVAL,
                                  "uv_os_get_passwd",
                                  "Invalid character encoding for username");
   }
 
   if (homedir.IsEmpty()) {
+    // TODO(addaleax): Use `error` itself here.
     return env->ThrowUVException(UV_EINVAL,
                                  "uv_os_get_passwd",
                                  "Invalid character encoding for homedir");
   }
 
   if (shell.IsEmpty()) {
+    // TODO(addaleax): Use `error` itself here.
     return env->ThrowUVException(UV_EINVAL,
                                  "uv_os_get_passwd",
                                  "Invalid character encoding for shell");
@@ -363,9 +400,9 @@ static void GetUserInfo(const FunctionCallbackInfo<Value>& args) {
 
   entry->Set(env->uid_string(), uid);
   entry->Set(env->gid_string(), gid);
-  entry->Set(env->username_string(), username);
-  entry->Set(env->homedir_string(), homedir);
-  entry->Set(env->shell_string(), shell);
+  entry->Set(env->username_string(), username.ToLocalChecked());
+  entry->Set(env->homedir_string(), homedir.ToLocalChecked());
+  entry->Set(env->shell_string(), shell.ToLocalChecked());
 
   args.GetReturnValue().Set(entry);
 }
@@ -393,4 +430,4 @@ void Initialize(Local<Object> target,
 }  // namespace os
 }  // namespace node
 
-NODE_MODULE_CONTEXT_AWARE_BUILTIN(os, node::os::Initialize)
+NODE_BUILTIN_MODULE_CONTEXT_AWARE(os, node::os::Initialize)
