@@ -1,6 +1,6 @@
 'use strict';
 const common = require('../../common');
-var assert = require('assert');
+const assert = require('assert');
 const binding = require(`./build/${common.buildType}/binding`);
-assert.equal('world', binding.hello());
+assert.strictEqual(binding.hello(), 'world');
 console.log('binding.hello() =', binding.hello());
