@@ -1,7 +1,4 @@
-#include "node.h"
-#include "../../../src/util.h"
-#include "../../../src/util-inl.h"
-
+#include <node.h>
 #include <assert.h>
 #include <openssl/rand.h>
 
@@ -32,4 +29,4 @@ inline void Initialize(v8::Local<v8::Object> exports,
 
 }  // anonymous namespace
 
-NODE_MODULE_CONTEXT_AWARE(binding, Initialize)
+NODE_MODULE_CONTEXT_AWARE(NODE_GYP_MODULE_NAME, Initialize)
