@@ -1,38 +1,34 @@
 'use strict';
 
 const common = require('../../common');
+const skipMessage = 'intensive toString tests due to memory confinements';
+if (!common.enoughTestMem)
+  common.skip(skipMessage);
+
 const binding = require(`./build/${common.buildType}/binding`);
 const assert = require('assert');
 
-const skipMessage = 'intensive toString tests due to memory confinements';
-if (!common.enoughTestMem) {
-  common.skip(skipMessage);
-  return;
-}
-
 // v8 fails silently if string length > v8::String::kMaxLength
 // v8::String::kMaxLength defined in v8.h
 const kStringMaxLength = process.binding('buffer').kStringMaxLength;
 
+let buf;
 try {
-  var buf = new Buffer(kStringMaxLength + 1);
+  buf = Buffer.allocUnsafe(kStringMaxLength + 1);
 } catch (e) {
   // If the exception is not due to memory confinement then rethrow it.
-  if (e.message !== 'Invalid array buffer length') throw (e);
+  if (e.message !== 'Array buffer allocation failed') throw (e);
   common.skip(skipMessage);
-  return;
 }
 
 // Ensure we have enough memory available for future allocations to succeed.
-if (!binding.ensureAllocation(2 * kStringMaxLength)) {
+if (!binding.ensureAllocation(2 * kStringMaxLength))
   common.skip(skipMessage);
-  return;
-}
 
 assert.throws(function() {
   buf.toString();
-}, /toString failed|Invalid array buffer length/);
+}, /"toString\(\)" failed|Array buffer allocation failed/);
 
 assert.throws(function() {
   buf.toString('utf8');
-}, /toString failed/);
+}, /"toString\(\)" failed/);
