@@ -2,22 +2,22 @@
 // just like test/gc/http-client.js,
 // but aborting every connection that comes in.
 
+const common = require('../common');
+
 function serverHandler(req, res) {
   res.connection.destroy();
 }
 
-const http  = require('http');
-const weak = require('weak');
-require('../common');
-const assert = require('assert');
+const http = require('http');
+const weak = require(`./build/${common.buildType}/binding`);
 const todo = 500;
 let done = 0;
 let count = 0;
 let countGC = 0;
 
-console.log('We should do ' + todo + ' requests');
+console.log(`We should do ${todo} requests`);
 
-var server = http.createServer(serverHandler);
+const server = http.createServer(serverHandler);
 server.listen(0, getall);
 
 function getall() {
@@ -27,10 +27,9 @@ function getall() {
   (function() {
     function cb(res) {
       done += 1;
-      statusLater();
     }
 
-    var req = http.get({
+    const req = http.get({
       hostname: 'localhost',
       pathname: '/',
       port: server.address().port
@@ -43,27 +42,18 @@ function getall() {
   setImmediate(getall);
 }
 
-for (var i = 0; i < 10; i++)
+for (let i = 0; i < 10; i++)
   getall();
 
 function afterGC() {
   countGC++;
 }
 
-var timer;
-function statusLater() {
-  gc();
-  if (timer) clearTimeout(timer);
-  timer = setTimeout(status, 1);
-}
+setInterval(status, 100).unref();
 
 function status() {
-  gc();
+  global.gc();
   console.log('Done: %d/%d', done, todo);
   console.log('Collected: %d/%d', countGC, count);
-  if (done === todo) {
-    console.log('All should be collected now.');
-    assert(count === countGC);
-    process.exit(0);
-  }
+  if (countGC === todo) server.close();
 }
