@@ -19,27 +19,16 @@
 // OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
 // USE OR OTHER DEALINGS IN THE SOFTWARE.
 
-var common = require('../common');
-var assert = require('assert');
-var net = require('net');
+'use strict';
+const common = require('../common');
+const net = require('net');
 
-var client, killed = false, ended = false;
-var TIMEOUT = 10 * 1000
+const TIMEOUT = 10 * 1000;
 
-client = net.createConnection(53, '8.8.8.8', function() {
+const client = net.createConnection(53, '8.8.8.8', function() {
   client.unref();
 });
 
-client.on('close', function() {
-  ended = true;
-});
+client.on('close', common.mustNotCall());
 
-setTimeout(function() {
-  killed = true;
-  client.end();
-}, TIMEOUT).unref();
-
-process.on('exit', function() {
-  assert.strictEqual(killed, false, 'A client should have connected');
-  assert.strictEqual(ended, false, 'A client should stay connected');
-});
+setTimeout(common.mustNotCall(), TIMEOUT).unref();
