@@ -1,17 +1,16 @@
 'use strict';
 const common = require('../common');
+// Fails with EINVAL on SmartOS, EBUSY on Windows, EBUSY on AIX.
+if (common.isSunOS || common.isWindows || common.isAIX)
+  common.skip('cannot rmdir current working directory');
+
 const assert = require('assert');
 const fs = require('fs');
 const spawn = require('child_process').spawn;
+const fixtures = require('../common/fixtures');
 
-// Fails with EINVAL on SmartOS, EBUSY on Windows, EBUSY on AIX.
-if (process.platform === 'sunos' || common.isWindows || common.isAix) {
-  console.log('1..0 # Skipped: cannot rmdir current working directory');
-  return;
-}
-
-const dirname = common.tmpDir + '/cwd-does-not-exist-' + process.pid;
-const abspathFile = require('path').join(common.fixturesDir, 'a.js');
+const dirname = `${common.tmpDir}/cwd-does-not-exist-${process.pid}`;
+const abspathFile = fixtures.path('a.js');
 common.refreshTmpDir();
 fs.mkdirSync(dirname);
 process.chdir(dirname);
