@@ -2,6 +2,7 @@
 const common = require('../common');
 const assert = require('assert');
 const http = require('http');
+const Countdown = require('../common/countdown');
 
 const MAX_SOCKETS = 2;
 
@@ -12,32 +13,29 @@ const agent = new http.Agent({
   maxFreeSockets: 2
 });
 
-const server = http.createServer(function(req, res) {
+const server = http.createServer(common.mustCall((req, res) => {
   res.end('hello world');
-});
+}, 6));
+
+const countdown = new Countdown(6, common.mustCall(() => server.close()));
 
 function get(path, callback) {
   return http.get({
     host: 'localhost',
-    port: common.PORT,
+    port: server.address().port,
     agent: agent,
     path: path
   }, callback);
 }
 
-server.listen(common.PORT, function() {
-  var finished = 0;
-  const num_requests = 6;
-  for (var i = 0; i < num_requests; i++) {
-    const request = get('/1', function() {
-    });
-    request.on('response', function() {
+server.listen(0, common.mustCall(() => {
+  for (let i = 0; i < 6; i++) {
+    const request = get('/1', common.mustCall());
+    request.on('response', common.mustCall(() => {
       request.abort();
       const sockets = agent.sockets[Object.keys(agent.sockets)[0]];
       assert(sockets.length <= MAX_SOCKETS);
-      if (++finished === num_requests) {
-        server.close();
-      }
-    });
+      countdown.dec();
+    }));
   }
-});
+}));
