@@ -1,5 +1,27 @@
+// Copyright Joyent, Inc. and other Node contributors.
+//
+// Permission is hereby granted, free of charge, to any person obtaining a
+// copy of this software and associated documentation files (the
+// "Software"), to deal in the Software without restriction, including
+// without limitation the rights to use, copy, modify, merge, publish,
+// distribute, sublicense, and/or sell copies of the Software, and to permit
+// persons to whom the Software is furnished to do so, subject to the
+// following conditions:
+//
+// The above copyright notice and this permission notice shall be included
+// in all copies or substantial portions of the Software.
+//
+// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
+// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
+// NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
+// DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
+// OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
+// USE OR OTHER DEALINGS IN THE SOFTWARE.
+
 'use strict';
 const common = require('../common');
+const fixtures = require('../common/fixtures');
 if (!common.hasCrypto)
   common.skip('missing crypto');
 
@@ -8,15 +30,9 @@ process.env.NODE_TLS_REJECT_UNAUTHORIZED = '0';
 
 const assert = require('assert');
 const https = require('https');
-const fs = require('fs');
-const path = require('path');
-
-function file(fname) {
-  return path.resolve(common.fixturesDir, 'keys', fname);
-}
 
 function read(fname) {
-  return fs.readFileSync(file(fname));
+  return fixtures.readKey(fname);
 }
 
 // key1 is signed by ca1.
@@ -67,12 +83,10 @@ server2.listen(0, listening());
 server3.listen(0, listening());
 
 const responseErrors = {};
-let expectResponseCount = 0;
-let responseCount = 0;
 let pending = 0;
 
 
-function server(options, port) {
+function server(options) {
   const s = https.createServer(options, handler);
   s.requests = [];
   s.expectCount = 0;
@@ -88,7 +102,7 @@ function handler(req, res) {
 
 function listening() {
   listenWait++;
-  return function() {
+  return () => {
     listenWait--;
     if (listenWait === 0) {
       allListening();
@@ -98,11 +112,7 @@ function listening() {
 
 function makeReq(path, port, error, host, ca) {
   pending++;
-  const options = {
-    port: port,
-    path: path,
-    ca: ca
-  };
+  const options = { port, path, ca };
 
   if (!ca) {
     options.agent = agent0;
@@ -120,20 +130,17 @@ function makeReq(path, port, error, host, ca) {
   }
 
   if (host) {
-    options.headers = { host: host };
+    options.headers = { host };
   }
   const req = https.get(options);
-  expectResponseCount++;
-  const server = port === server1.address().port ? server1
-    : port === server2.address().port ? server2
-      : port === server3.address().port ? server3
-        : null;
-
+  const server = port === server1.address().port ? server1 :
+    port === server2.address().port ? server2 :
+      port === server3.address().port ? server3 :
+        null;
   if (!server) throw new Error(`invalid port: ${port}`);
   server.expectCount++;
 
-  req.on('response', function(res) {
-    responseCount++;
+  req.on('response', common.mustCall((res) => {
     assert.strictEqual(res.connection.authorizationError, error);
     responseErrors[path] = res.connection.authorizationError;
     pending--;
@@ -143,7 +150,7 @@ function makeReq(path, port, error, host, ca) {
       server3.close();
     }
     res.resume();
-  });
+  }));
 }
 
 function allListening() {
@@ -192,10 +199,8 @@ function allListening() {
 
 }
 
-process.on('exit', function() {
-  console.error(responseErrors);
+process.on('exit', () => {
   assert.strictEqual(server1.requests.length, server1.expectCount);
   assert.strictEqual(server2.requests.length, server2.expectCount);
   assert.strictEqual(server3.requests.length, server3.expectCount);
-  assert.strictEqual(responseCount, expectResponseCount);
 });
