@@ -1,5 +1,5 @@
 'use strict';
-const common = require('../common');
+require('../common');
 const assert = require('assert');
 const domain = require('domain');
 
@@ -11,9 +11,9 @@ const asyncTest = (function() {
   let currentTest = null;
 
   function fail(error) {
-    const stack = currentTest
-      ? `${error.stack}\nFrom previous event:\n${currentTest.stack}`
-      : error.stack;
+    const stack = currentTest ?
+      `${error.stack}\nFrom previous event:\n${currentTest.stack}` :
+      error.stack;
 
     if (currentTest)
       process.stderr.write(`'${currentTest.description}' failed\n\n`);
@@ -164,7 +164,7 @@ asyncTest('Catching a promise rejection after setImmediate is not' +
   });
   _reject(e);
   setImmediate(function() {
-    promise.then(common.fail, () => {});
+    promise.then(assert.fail, function() {});
   });
 });
 
@@ -176,13 +176,13 @@ asyncTest('When re-throwing new errors in a promise catch, only the' +
     assert.strictEqual(e2, reason);
     assert.strictEqual(promise2, promise);
   });
-  let promise2 = Promise.reject(e).then(common.fail, function(reason) {
+  const promise2 = Promise.reject(e).then(assert.fail, function(reason) {
     assert.strictEqual(e, reason);
     throw e2;
   });
 });
 
-asyncTest('Test params of unhandledRejection for a synchronously-rejected' +
+asyncTest('Test params of unhandledRejection for a synchronously-rejected ' +
           'promise', function(done) {
   const e = new Error();
   onUnhandledSucceed(done, function(reason, promise) {
@@ -201,11 +201,11 @@ asyncTest('When re-throwing new errors in a promise catch, only the ' +
     assert.strictEqual(e2, reason);
     assert.strictEqual(promise2, promise);
   });
-  let promise2 = new Promise(function(_, reject) {
+  const promise2 = new Promise(function(_, reject) {
     setTimeout(function() {
       reject(e);
     }, 1);
-  }).then(common.fail, function(reason) {
+  }).then(assert.fail, function(reason) {
     assert.strictEqual(e, reason);
     throw e2;
   });
@@ -224,7 +224,7 @@ asyncTest('When re-throwing new errors in a promise catch, only the re-thrown' +
     setTimeout(function() {
       reject(e);
       process.nextTick(function() {
-        promise2 = promise.then(common.fail, function(reason) {
+        promise2 = promise.then(assert.fail, function(reason) {
           assert.strictEqual(e, reason);
           throw e2;
         });
@@ -240,7 +240,7 @@ asyncTest(
   function(done) {
     const e = new Error();
     onUnhandledFail(done);
-    Promise.reject(e).then(common.fail, () => {});
+    Promise.reject(e).then(assert.fail, function() {});
   }
 );
 
@@ -252,7 +252,7 @@ asyncTest(
     onUnhandledFail(done);
     new Promise(function(_, reject) {
       reject(e);
-    }).then(common.fail, () => {});
+    }).then(assert.fail, function() {});
   }
 );
 
@@ -262,7 +262,7 @@ asyncTest('Attaching a promise catch in a process.nextTick is soon enough to' +
   onUnhandledFail(done);
   const promise = Promise.reject(e);
   process.nextTick(function() {
-    promise.then(common.fail, () => {});
+    promise.then(assert.fail, function() {});
   });
 });
 
@@ -274,7 +274,7 @@ asyncTest('Attaching a promise catch in a process.nextTick is soon enough to' +
     reject(e);
   });
   process.nextTick(function() {
-    promise.then(common.fail, () => {});
+    promise.then(assert.fail, function() {});
   });
 });
 
@@ -294,7 +294,7 @@ asyncTest('While inside setImmediate, catching a rejected promise derived ' +
 });
 
 // State adapation tests
-asyncTest('catching a promise which is asynchronously rejected (via' +
+asyncTest('catching a promise which is asynchronously rejected (via ' +
           'resolution to an asynchronously-rejected promise) prevents' +
           ' unhandledRejection', function(done) {
   const e = new Error();
@@ -305,7 +305,7 @@ asyncTest('catching a promise which is asynchronously rejected (via' +
         reject(e);
       }, 1);
     });
-  }).then(common.fail, function(reason) {
+  }).then(assert.fail, function(reason) {
     assert.strictEqual(e, reason);
   });
 });
@@ -316,7 +316,7 @@ asyncTest('Catching a rejected promise derived from throwing in a' +
   onUnhandledFail(done);
   Promise.resolve().then(function() {
     throw e;
-  }).then(common.fail, function(reason) {
+  }).then(assert.fail, function(reason) {
     assert.strictEqual(e, reason);
   });
 });
@@ -328,7 +328,7 @@ asyncTest('Catching a rejected promise derived from returning a' +
   onUnhandledFail(done);
   Promise.resolve().then(function() {
     return Promise.reject(e);
-  }).then(common.fail, function(reason) {
+  }).then(assert.fail, function(reason) {
     assert.strictEqual(e, reason);
   });
 });
@@ -337,12 +337,11 @@ asyncTest('A rejected promise derived from returning an' +
           ' asynchronously-rejected promise in a fulfillment handler' +
           ' does trigger unhandledRejection', function(done) {
   const e = new Error();
-  let _promise;
   onUnhandledSucceed(done, function(reason, promise) {
     assert.strictEqual(e, reason);
     assert.strictEqual(_promise, promise);
   });
-  _promise = Promise.resolve().then(function() {
+  const _promise = Promise.resolve().then(function() {
     return new Promise(function(_, reject) {
       setTimeout(function() {
         reject(e);
@@ -354,12 +353,11 @@ asyncTest('A rejected promise derived from returning an' +
 asyncTest('A rejected promise derived from throwing in a fulfillment handler' +
           ' does trigger unhandledRejection', function(done) {
   const e = new Error();
-  let _promise;
   onUnhandledSucceed(done, function(reason, promise) {
     assert.strictEqual(e, reason);
     assert.strictEqual(_promise, promise);
   });
-  _promise = Promise.resolve().then(function() {
+  const _promise = Promise.resolve().then(function() {
     throw e;
   });
 });
@@ -369,23 +367,22 @@ asyncTest(
   ' promise in a fulfillment handler does trigger unhandledRejection',
   function(done) {
     const e = new Error();
-    let _promise;
     onUnhandledSucceed(done, function(reason, promise) {
       assert.strictEqual(e, reason);
       assert.strictEqual(_promise, promise);
     });
-    _promise = Promise.resolve().then(function() {
+    const _promise = Promise.resolve().then(function() {
       return Promise.reject(e);
     });
   }
 );
 
 // Combinations with Promise.all
-asyncTest('Catching the Promise.all() of a collection that includes a' +
+asyncTest('Catching the Promise.all() of a collection that includes a ' +
           'rejected promise prevents unhandledRejection', function(done) {
   const e = new Error();
   onUnhandledFail(done);
-  Promise.all([Promise.reject(e)]).then(common.fail, () => {});
+  Promise.all([Promise.reject(e)]).then(assert.fail, function() {});
 });
 
 asyncTest(
@@ -401,7 +398,7 @@ asyncTest(
     });
     p = Promise.all([p]);
     process.nextTick(function() {
-      p.then(common.fail, () => {});
+      p.then(assert.fail, function() {});
     });
   }
 );
@@ -414,7 +411,7 @@ asyncTest('Failing to catch the Promise.all() of a collection that includes' +
     assert.strictEqual(e, reason);
     assert.strictEqual(p, promise);
   });
-  let p = Promise.all([Promise.reject(e)]);
+  const p = Promise.all([Promise.reject(e)]);
 });
 
 asyncTest('Waiting setTimeout(, 10) to catch a promise causes an' +
@@ -433,11 +430,11 @@ asyncTest('Waiting setTimeout(, 10) to catch a promise causes an' +
     done();
   });
 
-  let thePromise = new Promise(function() {
+  const thePromise = new Promise(function() {
     throw e;
   });
   setTimeout(function() {
-    thePromise.then(common.fail, function(reason) {
+    thePromise.then(assert.fail, function(reason) {
       assert.strictEqual(e, reason);
     });
   }, 10);
@@ -455,7 +452,7 @@ asyncTest('Waiting for some combination of process.nextTick + promise' +
     Promise.resolve().then(function() {
       process.nextTick(function() {
         Promise.resolve().then(function() {
-          a.catch(() => {});
+          a.catch(function() {});
         });
       });
     });
@@ -474,7 +471,7 @@ asyncTest('Waiting for some combination of process.nextTick + promise' +
       Promise.resolve().then(function() {
         process.nextTick(function() {
           Promise.resolve().then(function() {
-            a.catch(() => {});
+            a.catch(function() {});
           });
         });
       });
@@ -494,7 +491,7 @@ asyncTest('Waiting for some combination of process.nextTick + promise ' +
       Promise.resolve().then(function() {
         process.nextTick(function() {
           Promise.resolve().then(function() {
-            a.catch(() => {});
+            a.catch(function() {});
           });
         });
       });
@@ -514,7 +511,7 @@ asyncTest('Waiting for some combination of promise microtasks + ' +
     process.nextTick(function() {
       Promise.resolve().then(function() {
         process.nextTick(function() {
-          a.catch(() => {});
+          a.catch(function() {});
         });
       });
     });
@@ -535,7 +532,7 @@ asyncTest(
         process.nextTick(function() {
           Promise.resolve().then(function() {
             process.nextTick(function() {
-              a.catch(() => {});
+              a.catch(function() {});
             });
           });
         });
@@ -556,7 +553,7 @@ asyncTest('Waiting for some combination of promise microtasks +' +
       process.nextTick(function() {
         Promise.resolve().then(function() {
           process.nextTick(function() {
-            a.catch(() => {});
+            a.catch(function() {});
           });
         });
       });
@@ -572,10 +569,10 @@ asyncTest('setImmediate + promise microtasks is too late to attach a catch' +
     assert.strictEqual(e, reason);
     assert.strictEqual(p, promise);
   });
-  let p = Promise.reject(e);
+  const p = Promise.reject(e);
   setImmediate(function() {
     Promise.resolve().then(function() {
-      p.catch(() => {});
+      p.catch(function() {});
     });
   });
 });
@@ -592,13 +589,13 @@ asyncTest('setImmediate + promise microtasks is too late to attach a catch' +
       Promise.resolve().then(function() {
         Promise.resolve().then(function() {
           Promise.resolve().then(function() {
-            p.catch(() => {});
+            p.catch(function() {});
           });
         });
       });
     });
   });
-  let p = Promise.reject();
+  const p = Promise.reject();
 });
 
 asyncTest('setImmediate + promise microtasks is too late to attach a catch' +
@@ -608,13 +605,13 @@ asyncTest('setImmediate + promise microtasks is too late to attach a catch' +
     assert.strictEqual(undefined, reason);
     assert.strictEqual(p, promise);
   });
-  let p = Promise.reject();
+  const p = Promise.reject();
   setImmediate(function() {
     Promise.resolve().then(function() {
       Promise.resolve().then(function() {
         Promise.resolve().then(function() {
           Promise.resolve().then(function() {
-            p.catch(() => {});
+            p.catch(function() {});
           });
         });
       });
@@ -666,7 +663,7 @@ asyncTest('nextTick is immediately scheduled when called inside an event' +
 });
 
 asyncTest('Throwing an error inside a rejectionHandled handler goes to' +
-          ' unhandledException, and does not cause .catch() to throw an' +
+          ' unhandledException, and does not cause .catch() to throw an ' +
           'exception', function(done) {
   clean();
   const e = new Error();
@@ -682,7 +679,7 @@ asyncTest('Throwing an error inside a rejectionHandled handler goes to' +
   const p = Promise.reject(e);
   setTimeout(function() {
     try {
-      p.catch(() => {});
+      p.catch(function() {});
     } catch (e) {
       done(new Error('fail'));
     }
