@@ -1,19 +1,40 @@
+// Copyright Joyent, Inc. and other Node contributors.
+//
+// Permission is hereby granted, free of charge, to any person obtaining a
+// copy of this software and associated documentation files (the
+// "Software"), to deal in the Software without restriction, including
+// without limitation the rights to use, copy, modify, merge, publish,
+// distribute, sublicense, and/or sell copies of the Software, and to permit
+// persons to whom the Software is furnished to do so, subject to the
+// following conditions:
+//
+// The above copyright notice and this permission notice shall be included
+// in all copies or substantial portions of the Software.
+//
+// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
+// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
+// NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
+// DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
+// OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
+// USE OR OTHER DEALINGS IN THE SOFTWARE.
+
+// Flags: --expose_internals
 'use strict';
 const common = require('../common');
+
 const assert = require('assert');
 const readline = require('readline');
+const internalReadline = require('internal/readline');
 const EventEmitter = require('events').EventEmitter;
-const inherits = require('util').inherits;
 const { Writable, Readable } = require('stream');
 
-function FakeInput() {
-  EventEmitter.call(this);
+class FakeInput extends EventEmitter {
+  resume() {}
+  pause() {}
+  write() {}
+  end() {}
 }
-inherits(FakeInput, EventEmitter);
-FakeInput.prototype.resume = () => {};
-FakeInput.prototype.pause = () => {};
-FakeInput.prototype.write = () => {};
-FakeInput.prototype.end = () => {};
 
 function isWarned(emitter) {
   for (const name in emitter) {
@@ -34,7 +55,7 @@ function isWarned(emitter) {
 {
   // Minimum crlfDelay is 100ms
   const fi = new FakeInput();
-  const rli = new readline.Interface({ input: fi, output: fi, crlfDelay: 0});
+  const rli = new readline.Interface({ input: fi, output: fi, crlfDelay: 0 });
   assert.strictEqual(rli.crlfDelay, 100);
   rli.close();
 }
@@ -52,6 +73,7 @@ function isWarned(emitter) {
 }
 
 {
+  // set crlfDelay to 5000ms
   const fi = new FakeInput();
   const rli = new readline.Interface({
     input: fi,
@@ -209,43 +231,6 @@ function isWarned(emitter) {
   // sending multiple newlines at once that does not end with a new line
   // and a `end` event(last line is)
 
-  // \r\n should emit one line event, not two
-  {
-    const fi = new FakeInput();
-    const rli = new readline.Interface(
-      { input: fi, output: fi, terminal: terminal }
-    );
-    const expectedLines = ['foo', 'bar', 'baz', 'bat'];
-    let callCount = 0;
-    rli.on('line', function(line) {
-      assert.strictEqual(line, expectedLines[callCount]);
-      callCount++;
-    });
-    fi.emit('data', expectedLines.join('\r\n'));
-    assert.strictEqual(callCount, expectedLines.length - 1);
-    rli.close();
-  }
-
-  // \r\n should emit one line event when split across multiple writes.
-  {
-    const fi = new FakeInput();
-    const rli = new readline.Interface(
-      { input: fi, output: fi, terminal: terminal }
-    );
-    const expectedLines = ['foo', 'bar', 'baz', 'bat'];
-    let callCount = 0;
-    rli.on('line', function(line) {
-      assert.strictEqual(line, expectedLines[callCount]);
-      callCount++;
-    });
-    expectedLines.forEach(function(line) {
-      fi.emit('data', `${line}\r`);
-      fi.emit('data', '\n');
-    });
-    assert.strictEqual(callCount, expectedLines.length);
-    rli.close();
-  }
-
   // \r should behave like \n when alone
   {
     const fi = new FakeInput();
@@ -303,30 +288,6 @@ function isWarned(emitter) {
     }), delay * 2);
   }
 
-  // Emit one line events when the delay between \r and \n is
-  // over the default crlfDelay but within the setting value
-  {
-    const fi = new FakeInput();
-    const delay = 125;
-    const crlfDelay = common.platformTimeout(1000);
-    const rli = new readline.Interface({
-      input: fi,
-      output: fi,
-      terminal: terminal,
-      crlfDelay
-    });
-    let callCount = 0;
-    rli.on('line', function(line) {
-      callCount++;
-    });
-    fi.emit('data', '\r');
-    setTimeout(common.mustCall(() => {
-      fi.emit('data', '\n');
-      assert.strictEqual(callCount, 1);
-      rli.close();
-    }), delay);
-  }
-
   // set crlfDelay to `Infinity` is allowed
   {
     const fi = new FakeInput();
@@ -411,6 +372,37 @@ function isWarned(emitter) {
     });
   }
 
+  // constructor throws if historySize is not a positive number
+  {
+    const fi = new FakeInput();
+    assert.throws(function() {
+      readline.createInterface({
+        input: fi, historySize: 'not a number'
+      });
+    }, common.expectsError({
+      type: TypeError,
+      message: 'Argument "historySize" must be a positive number'
+    }));
+
+    assert.throws(function() {
+      readline.createInterface({
+        input: fi, historySize: -1
+      });
+    }, common.expectsError({
+      type: TypeError,
+      message: 'Argument "historySize" must be a positive number'
+    }));
+
+    assert.throws(function() {
+      readline.createInterface({
+        input: fi, historySize: NaN
+      });
+    }, common.expectsError({
+      type: TypeError,
+      message: 'Argument "historySize" must be a positive number'
+    }));
+  }
+
   // duplicate lines are removed from history when
   // `options.removeHistoryDuplicates` is `true`
   {
@@ -440,6 +432,14 @@ function isWarned(emitter) {
     assert.notStrictEqual(rli.line, expectedLines[--callCount]);
     assert.strictEqual(rli.line, expectedLines[--callCount]);
     assert.strictEqual(callCount, 0);
+    fi.emit('keypress', '.', { name: 'down' }); // 'baz'
+    assert.strictEqual(rli.line, 'baz');
+    fi.emit('keypress', '.', { name: 'n', ctrl: true }); // 'bar'
+    assert.strictEqual(rli.line, 'bar');
+    fi.emit('keypress', '.', { name: 'down' }); // 'bat'
+    assert.strictEqual(rli.line, 'bat');
+    fi.emit('keypress', '.', { name: 'down' }); // ''
+    assert.strictEqual(rli.line, '');
     rli.close();
   }
 
@@ -535,7 +535,35 @@ function isWarned(emitter) {
     rli.close();
   }
 
+  // calling the question callback
+  {
+    let called = false;
+    const fi = new FakeInput();
+    const rli = new readline.Interface(
+      { input: fi, output: fi, terminal: terminal }
+    );
+    rli.question('foo?', function(answer) {
+      called = true;
+      assert.strictEqual(answer, 'bar');
+    });
+    rli.write('bar\n');
+    assert.ok(called);
+    rli.close();
+  }
+
   if (terminal) {
+    // history is bound
+    {
+      const fi = new FakeInput();
+      const rli = new readline.Interface(
+        { input: fi, output: fi, terminal, historySize: 2 }
+      );
+      const lines = ['line 1', 'line 2', 'line 3'];
+      fi.emit('data', lines.join('\n') + '\n');
+      assert.strictEqual(rli.history.length, 2);
+      assert.strictEqual(rli.history[0], 'line 3');
+      assert.strictEqual(rli.history[1], 'line 2');
+    }
     // question
     {
       const fi = new FakeInput();
@@ -567,54 +595,206 @@ function isWarned(emitter) {
       assert.strictEqual(cursorPos.cols, expectedLines.slice(-1)[0].length);
       rli.close();
     }
+
+    {
+      // Beginning and end of line
+      const fi = new FakeInput();
+      const rli = new readline.Interface({
+        input: fi,
+        output: fi,
+        prompt: '',
+        terminal: terminal
+      });
+      fi.emit('data', 'the quick brown fox');
+      fi.emit('keypress', '.', { ctrl: true, name: 'a' });
+      let cursorPos = rli._getCursorPos();
+      assert.strictEqual(cursorPos.rows, 0);
+      assert.strictEqual(cursorPos.cols, 0);
+      fi.emit('keypress', '.', { ctrl: true, name: 'e' });
+      cursorPos = rli._getCursorPos();
+      assert.strictEqual(cursorPos.rows, 0);
+      assert.strictEqual(cursorPos.cols, 19);
+      rli.close();
+    }
+
+    {
+      // `wordLeft` and `wordRight`
+      const fi = new FakeInput();
+      const rli = new readline.Interface({
+        input: fi,
+        output: fi,
+        prompt: '',
+        terminal: terminal
+      });
+      fi.emit('data', 'the quick brown fox');
+      fi.emit('keypress', '.', { ctrl: true, name: 'left' });
+      let cursorPos = rli._getCursorPos();
+      assert.strictEqual(cursorPos.rows, 0);
+      assert.strictEqual(cursorPos.cols, 16);
+      fi.emit('keypress', '.', { meta: true, name: 'b' });
+      cursorPos = rli._getCursorPos();
+      assert.strictEqual(cursorPos.rows, 0);
+      assert.strictEqual(cursorPos.cols, 10);
+      fi.emit('keypress', '.', { ctrl: true, name: 'right' });
+      cursorPos = rli._getCursorPos();
+      assert.strictEqual(cursorPos.rows, 0);
+      assert.strictEqual(cursorPos.cols, 16);
+      fi.emit('keypress', '.', { meta: true, name: 'f' });
+      cursorPos = rli._getCursorPos();
+      assert.strictEqual(cursorPos.rows, 0);
+      assert.strictEqual(cursorPos.cols, 19);
+      rli.close();
+    }
+
+    {
+      // `deleteWordLeft`
+      [
+        { ctrl: true, name: 'w' },
+        { ctrl: true, name: 'backspace' },
+        { meta: true, name: 'backspace' }
+      ]
+        .forEach((deleteWordLeftKey) => {
+          let fi = new FakeInput();
+          let rli = new readline.Interface({
+            input: fi,
+            output: fi,
+            prompt: '',
+            terminal: terminal
+          });
+          fi.emit('data', 'the quick brown fox');
+          fi.emit('keypress', '.', { ctrl: true, name: 'left' });
+          rli.on('line', common.mustCall((line) => {
+            assert.strictEqual(line, 'the quick fox');
+          }));
+          fi.emit('keypress', '.', deleteWordLeftKey);
+          fi.emit('data', '\n');
+          rli.close();
+
+          // No effect if pressed at beginning of line
+          fi = new FakeInput();
+          rli = new readline.Interface({
+            input: fi,
+            output: fi,
+            prompt: '',
+            terminal: terminal
+          });
+          fi.emit('data', 'the quick brown fox');
+          fi.emit('keypress', '.', { ctrl: true, name: 'a' });
+          rli.on('line', common.mustCall((line) => {
+            assert.strictEqual(line, 'the quick brown fox');
+          }));
+          fi.emit('keypress', '.', deleteWordLeftKey);
+          fi.emit('data', '\n');
+          rli.close();
+        });
+    }
+
+    {
+      // `deleteWordRight`
+      [
+        { ctrl: true, name: 'delete' },
+        { meta: true, name: 'delete' },
+        { meta: true, name: 'd' }
+      ]
+        .forEach((deleteWordRightKey) => {
+          let fi = new FakeInput();
+          let rli = new readline.Interface({
+            input: fi,
+            output: fi,
+            prompt: '',
+            terminal: terminal
+          });
+          fi.emit('data', 'the quick brown fox');
+          fi.emit('keypress', '.', { ctrl: true, name: 'left' });
+          fi.emit('keypress', '.', { ctrl: true, name: 'left' });
+          rli.on('line', common.mustCall((line) => {
+            assert.strictEqual(line, 'the quick fox');
+          }));
+          fi.emit('keypress', '.', deleteWordRightKey);
+          fi.emit('data', '\n');
+          rli.close();
+
+          // No effect if pressed at end of line
+          fi = new FakeInput();
+          rli = new readline.Interface({
+            input: fi,
+            output: fi,
+            prompt: '',
+            terminal: terminal
+          });
+          fi.emit('data', 'the quick brown fox');
+          rli.on('line', common.mustCall((line) => {
+            assert.strictEqual(line, 'the quick brown fox');
+          }));
+          fi.emit('keypress', '.', deleteWordRightKey);
+          fi.emit('data', '\n');
+          rli.close();
+        });
+    }
+
+    // multi-line cursor position
+    {
+      const fi = new FakeInput();
+      const rli = new readline.Interface({
+        input: fi,
+        output: fi,
+        prompt: '',
+        terminal: terminal
+      });
+      fi.columns = 10;
+      fi.emit('data', 'multi-line text');
+      const cursorPos = rli._getCursorPos();
+      assert.strictEqual(cursorPos.rows, 1);
+      assert.strictEqual(cursorPos.cols, 5);
+    }
   }
 
   // isFullWidthCodePoint() should return false for non-numeric values
   [true, false, null, undefined, {}, [], 'あ'].forEach((v) => {
-    assert.strictEqual(readline.isFullWidthCodePoint('あ'), false);
+    assert.strictEqual(internalReadline.isFullWidthCodePoint('あ'), false);
   });
 
   // wide characters should be treated as two columns.
-  assert.strictEqual(readline.isFullWidthCodePoint('a'.charCodeAt(0)), false);
-  assert.strictEqual(readline.isFullWidthCodePoint('あ'.charCodeAt(0)), true);
-  assert.strictEqual(readline.isFullWidthCodePoint('谢'.charCodeAt(0)), true);
-  assert.strictEqual(readline.isFullWidthCodePoint('고'.charCodeAt(0)), true);
-  assert.strictEqual(readline.isFullWidthCodePoint(0x1f251), true); // surrogate
-  assert.strictEqual(readline.codePointAt('ABC', 0), 0x41);
-  assert.strictEqual(readline.codePointAt('あいう', 1), 0x3044);
-  assert.strictEqual(readline.codePointAt('\ud800\udc00', 0),  // surrogate
-                     0x10000);
-  assert.strictEqual(readline.codePointAt('\ud800\udc00A', 2), // surrogate
-                     0x41);
-  assert.strictEqual(readline.getStringWidth('abcde'), 5);
-  assert.strictEqual(readline.getStringWidth('古池や'), 6);
-  assert.strictEqual(readline.getStringWidth('ノード.js'), 9);
-  assert.strictEqual(readline.getStringWidth('你好'), 4);
-  assert.strictEqual(readline.getStringWidth('안녕하세요'), 10);
-  assert.strictEqual(readline.getStringWidth('A\ud83c\ude00BC'),
-                     5); // surrogate
+  assert.strictEqual(internalReadline.isFullWidthCodePoint('a'.charCodeAt(0)),
+                     false);
+  assert.strictEqual(internalReadline.isFullWidthCodePoint('あ'.charCodeAt(0)),
+                     true);
+  assert.strictEqual(internalReadline.isFullWidthCodePoint('谢'.charCodeAt(0)),
+                     true);
+  assert.strictEqual(internalReadline.isFullWidthCodePoint('고'.charCodeAt(0)),
+                     true);
+  assert.strictEqual(internalReadline.isFullWidthCodePoint(0x1f251), true);
+  assert.strictEqual(internalReadline.getStringWidth('abcde'), 5);
+  assert.strictEqual(internalReadline.getStringWidth('古池や'), 6);
+  assert.strictEqual(internalReadline.getStringWidth('ノード.js'), 9);
+  assert.strictEqual(internalReadline.getStringWidth('你好'), 4);
+  assert.strictEqual(internalReadline.getStringWidth('안녕하세요'), 10);
+  assert.strictEqual(internalReadline.getStringWidth('A\ud83c\ude00BC'), 5);
 
   // check if vt control chars are stripped
   assert.strictEqual(
-    readline.stripVTControlCharacters('\u001b[31m> \u001b[39m'),
+    internalReadline.stripVTControlCharacters('\u001b[31m> \u001b[39m'),
     '> '
   );
   assert.strictEqual(
-    readline.stripVTControlCharacters('\u001b[31m> \u001b[39m> '),
+    internalReadline.stripVTControlCharacters('\u001b[31m> \u001b[39m> '),
     '> > '
   );
   assert.strictEqual(
-    readline.stripVTControlCharacters('\u001b[31m\u001b[39m'),
+    internalReadline.stripVTControlCharacters('\u001b[31m\u001b[39m'),
     ''
   );
   assert.strictEqual(
-    readline.stripVTControlCharacters('> '),
+    internalReadline.stripVTControlCharacters('> '),
     '> '
   );
-  assert.strictEqual(readline.getStringWidth('\u001b[31m> \u001b[39m'), 2);
-  assert.strictEqual(readline.getStringWidth('\u001b[31m> \u001b[39m> '), 4);
-  assert.strictEqual(readline.getStringWidth('\u001b[31m\u001b[39m'), 0);
-  assert.strictEqual(readline.getStringWidth('> '), 2);
+  assert.strictEqual(internalReadline
+    .getStringWidth('\u001b[31m> \u001b[39m'), 2);
+  assert.strictEqual(internalReadline
+    .getStringWidth('\u001b[31m> \u001b[39m> '), 4);
+  assert.strictEqual(internalReadline
+    .getStringWidth('\u001b[31m\u001b[39m'), 0);
+  assert.strictEqual(internalReadline.getStringWidth('> '), 2);
 
   {
     const fi = new FakeInput();
@@ -668,9 +848,9 @@ function isWarned(emitter) {
   }
 
   {
-    const expected = terminal
-      ? ['\u001b[1G', '\u001b[0J', '$ ', '\u001b[3G']
-      : ['$ '];
+    const expected = terminal ?
+      ['\u001b[1G', '\u001b[0J', '$ ', '\u001b[3G'] :
+      ['$ '];
 
     let counter = 0;
     const output = new Writable({
