@@ -4,22 +4,17 @@ const assert = require('assert');
 const path = require('path');
 const childProcess = require('child_process');
 const fs = require('fs');
+const fixtures = require('../common/fixtures');
 
-const scriptString = path.join(common.fixturesDir, 'print-chars.js');
-const scriptBuffer = path.join(common.fixturesDir,
-                               'print-chars-from-buffer.js');
+const scriptString = fixtures.path('print-chars.js');
+const scriptBuffer = fixtures.path('print-chars-from-buffer.js');
 const tmpFile = path.join(common.tmpDir, 'stdout.txt');
 
 common.refreshTmpDir();
 
 function test(size, useBuffer, cb) {
-  var cmd = '"' + process.argv[0] + '"' +
-            ' ' +
-            '"' + (useBuffer ? scriptBuffer : scriptString) + '"' +
-            ' ' +
-            size +
-            ' > ' +
-            '"' + tmpFile + '"';
+  const cmd = `"${process.argv[0]}" "${
+    useBuffer ? scriptBuffer : scriptString}" ${size} > "${tmpFile}"`;
 
   try {
     fs.unlinkSync(tmpFile);
@@ -28,11 +23,10 @@ function test(size, useBuffer, cb) {
   console.log(`${size} chars to ${tmpFile}...`);
 
   childProcess.exec(cmd, common.mustCall(function(err) {
-    if (err) throw err;
-
+    assert.ifError(err);
     console.log('done!');
 
-    var stat = fs.statSync(tmpFile);
+    const stat = fs.statSync(tmpFile);
 
     console.log(`${tmpFile} has ${stat.size} bytes`);
 
