@@ -10,7 +10,7 @@ const common = require('../common');
 const timers = require('timers');
 
 const foo = {
-  _onTimeout: common.fail
+  _onTimeout: common.mustNotCall('_onTimeout should not be called')
 };
 
 const bar = {
@@ -29,4 +29,4 @@ timers.enroll(foo, 50);
 timers._unrefActive(foo);
 
 // Keep the process open.
-setTimeout(function() {}, 100);
+setTimeout(() => {}, 100);
