@@ -1,5 +1,6 @@
 'use strict';
 const common = require('../common');
+const fixtures = require('../common/fixtures');
 
 if (!common.hasCrypto)
   common.skip('missing crypto');
@@ -7,18 +8,16 @@ if (!common.hasCrypto)
 const tls = require('tls');
 
 const assert = require('assert');
-const fs = require('fs');
-const path = require('path');
 
-const cert = fs.readFileSync(path.join(common.fixturesDir, 'test_cert.pem'));
-const key = fs.readFileSync(path.join(common.fixturesDir, 'test_key.pem'));
+const cert = fixtures.readSync('test_cert.pem');
+const key = fixtures.readSync('test_key.pem');
 
 // https://github.com/nodejs/node/issues/1489
 // tls.connect(options) with no options.host should accept a cert with
 //   CN:'localhost'
 tls.createServer({
-  key: key,
-  cert: cert
+  key,
+  cert
 }).listen(0, function() {
   const socket = tls.connect({
     port: this.address().port,
