@@ -13,10 +13,9 @@ const sslcontext = tls.createSecureContext({
   key: fixtures.readSync('test_key.pem')
 });
 
-let catchedServername;
 const pair = tls.createSecurePair(sslcontext, true, false, false, {
-  SNICallback: common.mustCall(function(servername, cb) {
-    catchedServername = servername;
+  SNICallback: common.mustCall((servername, cb) => {
+    assert.strictEqual(servername, 'www.google.com');
   })
 });
 
@@ -24,7 +23,3 @@ const pair = tls.createSecurePair(sslcontext, true, false, false, {
 const sslHello = fixtures.readSync('google_ssl_hello.bin');
 
 pair.encrypted.write(sslHello);
-
-process.on('exit', function() {
-  assert.strictEqual('www.google.com', catchedServername);
-});
