@@ -20,6 +20,6 @@ const res = vm.runInContext(code, o, 'test');
 assert(res);
 assert.strictEqual(typeof res, 'object');
 assert.strictEqual(res.value, 'val');
-assert.strictEqual(res.configurable, false, 'should not be configurable');
-assert.strictEqual(res.enumerable, false, 'should not be enumerable');
-assert.strictEqual(res.writable, false, 'should not be writable');
+assert.strictEqual(res.configurable, false);
+assert.strictEqual(res.enumerable, false);
+assert.strictEqual(res.writable, false);
